/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.components.jdbc;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.sql.DataSource;
import javax.xml.namespace.NamespaceContext;

import org.apache.commons.dbutils.DbUtils;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.Node;
import org.dom4j.dom.DOMDocument;
import org.eclipse.ebpm.components.jdbc.config.JDBCQueryConfig;
import org.eclipse.ebpm.components.jdbc.config.XmlResultSetHandler;
import org.eclipse.ebpm.messaging.api.Exchange;
import org.eclipse.ebpm.messaging.api.Message;
import org.eclipse.ebpm.xmlutil.DOM4JUtils;
import org.eclipse.ebpm.xmlutil.EXMLUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;



/**
 * Jdbc component that use the input NormalizedMessage to fill the store
 * procedure call statement.
 * <br>
 * The result is optionally inserted into a copy of the incoming message
 * otherwise a new message is create as output. The output should be formed
 * of more results in this case the call statement placeHolders are used
 * to rappresents output values on output message.

 *
 */
public class StoreProcedureComponent extends JDBCComponent {
	
	private static final Logger log = LoggerFactory.getLogger(StoreProcedureComponent.class);

	public boolean run(Exchange exchange, Message in, Message out)
	throws Exception {

		log.info("StoreProcedureComponent Component ["+getSpagicId()+"] process Exchange Method");	

		List<String> orderedParameterNameList = getStringParameterNames(queryConfig.getQuery(), queryConfig.getMatchParamRegexp());

		String jdbcQuery = queryConfig.getQuery().replaceAll(queryConfig.getMatchParamRegexp(), "?");
		
		List<org.eclipse.ebpm.components.jdbc.config.QueryParameterConfig> queryParametersConfig = queryConfig.getQueryParams();
		
		
		Object[] results = null;
		Connection conn = null;
		
		DataSource ds = getDataSourceManager().getDataSource(dsIdentifier);
		
		try {
				conn = ds.getConnection();
				StoreProcedureRunner spRunner = new StoreProcedureRunner();
				results = (Object[])spRunner.call(conn, 
												  jdbcQuery, 
												  in, 
												  queryParametersConfig,
												  orderedParameterNameList,
												  new XmlResultSetHandler(
														  	queryConfig.getRowsXmlEnvelope(),
														  	queryConfig.getRowXmlEnvelope())
					);
		} catch (SQLException sqle) {
			if (JDBCQueryConfig.FAULT_FLOW.equals(queryConfig.getFaultManagement())) {
				Message fault = exchange.getFault(true);
				Element faultElem = DocumentHelper.createElement("Fault");
				Element sqlStateElem = DocumentHelper.createElement("sql-state");
				sqlStateElem.setText(sqle.getSQLState());
				Element errorCodeElem = DocumentHelper.createElement("error-code");
				errorCodeElem.setText("" + sqle.getErrorCode());
				faultElem.add(sqlStateElem);
				faultElem.add(errorCodeElem);
				fault.setBody(faultElem.asXML());
				out.setBody(fault.getBody());
				return true;
			}
			throw sqle;
		} finally {
			DbUtils.close(conn);
		}
		
		//
		// compose out response. you need to manage single values and resultset
		// values (last ones were manages by ResultSetHandler)
		//
		out.setBody(in.getBody());
		String strContent = out.getBodyText();
		Document docContent = DocumentHelper.parseText(strContent);
		NamespaceContext nsContextMap = DOM4JUtils.getNamespaceContext(docContent);
		org.w3c.dom.Document docContentDOM = DOM4JUtils.dom4j_to_dom(docContent);
		EXMLUtils xmlUtils = new EXMLUtils(nsContextMap);
		
		
		
		
		
		// search the envelop where to put the result
		org.w3c.dom.Element contentEnvelop = null;

		if (isValidStr(queryConfig.getXmlEnvelope())) {
			contentEnvelop = xmlUtils.selectNode(docContentDOM,"/" + queryConfig.getXmlEnvelope());
			if (contentEnvelop == null) {
				log.warn("Envelope to enrich not found [" + "/" + queryConfig.getXmlEnvelope() + "]: "
						+ "the result will be the original message");
			}
		} else {
			log.info("Xml Envelope is null we will keep the original message ");
		}
		
		if (queryConfig.isEnrichMessage() && (contentEnvelop != null)) {
			
			
			if (queryConfig.isEnrichMessage() &&  results.length < 2) {
				//Element tmpResult = DocumentHelper.createElement("tmpResults");
				org.w3c.dom.Element tmpResult = docContentDOM.createElement("tmpResults");
				getResultsNodes(docContentDOM,results, tmpResult, orderedParameterNameList);
				((org.w3c.dom.Element)contentEnvelop).appendChild( tmpResult );
			} else {
				getResultsNodes(docContentDOM,results, (org.w3c.dom.Element)contentEnvelop, orderedParameterNameList);
			}
			out.setBody(docContent.asXML());
			
		} if (queryConfig.isEnrichMessage() && (contentEnvelop == null)) {
			out.setBody(docContent.asXML());
		} else {
			docContentDOM = xmlUtils.createBuilder().newDocument();
			
			String resultEnvelop = queryConfig.getXmlEnvelope();
			if (!isValidStr(resultEnvelop)){
				resultEnvelop = "ENVELOPE";
			}
			org.w3c.dom.Element el = docContentDOM.createElement(resultEnvelop);
			docContentDOM.appendChild(el);
			getResultsNodes(docContentDOM, results, docContentDOM.getDocumentElement(), orderedParameterNameList);
			out.setBody(docContent.asXML());
		}
		
		
		return true;
	}
	
	
	
	/**
	 * Return all results in dom4j elements where the element name is the
	 * placeHolder configured for the output parameter.
	 * 
	 * @param results
	 * @param parent
	 */
	private void getResultsNodes(org.w3c.dom.Document originalDoc, Object[] results, org.w3c.dom.Element parent, List<String> orderedParameterNameList) {
		Object elemObj = null;
		List<String> output = setOutputParameterNameList(orderedParameterNameList);
		for (int i = 0; i < results.length; i++) {
			elemObj = results[i];
			if (elemObj instanceof Element) {
				parent.appendChild((org.w3c.dom.Element)elemObj);
			} else {
				org.w3c.dom.Element elem = originalDoc.createElement(output.get(i));
				elem.setTextContent(elemObj != null? elemObj.toString() : "");
				parent.appendChild(elem);
			}
		}
	}
	
	
	
	private List<String> setOutputParameterNameList(List<String> orderedParameterNameList){
		List<String> orderedOutputParameterNameList = new ArrayList<String>();
		String paramName;
		for(int i=0; i < orderedParameterNameList.size(); i++){
			paramName = orderedParameterNameList.get(i);
			for(org.eclipse.ebpm.components.jdbc.config.QueryParameterConfig qpc : queryConfig.getQueryParams())
				if((qpc.getPlaceHolder().compareToIgnoreCase(paramName) == 0) && qpc.isOutputParam()){
					orderedOutputParameterNameList.add(paramName);
					break;
				}				
		}
		return orderedOutputParameterNameList;
	}

}
