/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.connectors.jms.marshaler;

import java.util.Enumeration;
import java.util.Set;

import javax.jms.JMSException;
import javax.jms.Message;

public abstract class AbstractJmsMarshaler {

    public static final String CONTENT_TYPE_PROPERTY = "SOAPJMS_contentType";

    /**
     * Should marshaler copy properties set in messages?
     */
    private boolean copyProperties = true;

    private boolean needJavaIdentifiers = true;

    /**
     * Get value from field copyProperties.
     * 
     * @return The copyProperties field value.
     */
    public boolean isCopyProperties() {
        return copyProperties;
    }

    /**
     * Set value from copyProperties field.
     * 
     * @param copyProperties New value for copyProperties field.
     */
    public void setCopyProperties(boolean copyProperties) {
        this.copyProperties = copyProperties;
    }

    /**
     * Copy properties from JMS message to JBI message.
     * 
     * @param message Received JMS message.
     * @param inMessage Created JBI message.
     * @throws JMSException If there is any problems with accessing to message
     *             properties.
     */
    @SuppressWarnings("unchecked")
    protected void copyPropertiesFromJMS(Message message, org.eclipse.ebpm.messaging.api.Message inMessage) throws JMSException {
        Enumeration<String> names = message.getPropertyNames();
        while (names.hasMoreElements()) {
            String name = names.nextElement();
            Object value = message.getObjectProperty(name);
			inMessage.setHeader(name, value);
        }
    }

    /**
     * Copy properties from JBI message to JMS message.
     * 
     * @param outMessage Received JBI message.
     * @param message Created JMS message.
     * @throws JMSException If there is any problems with saving JMS message
     *             properties.
     */
    protected void copyPropertiesFromNM(org.eclipse.ebpm.messaging.api.Message outMessage, Message message) throws JMSException {
        Set<String> names = outMessage.getHeaders().keySet();
        for (String name : names) {
            Object value = outMessage.getHeader(name);
			if (shouldIncludeHeader(name, value)) {
				message.setObjectProperty(name, value);
			}
        }
    }
    
    /**
     * checks whether the header property should be included or not
     * 
     * @param name the property name
     * @param value the property value
     * @return true if it should be copied
     */
    protected boolean shouldIncludeHeader(String name, Object value) {
        return (value instanceof Boolean || value instanceof Byte || value instanceof Short
                || value instanceof Integer || value instanceof Long || value instanceof Float
                || value instanceof Double || value instanceof String)
               && (!isNeedJavaIdentifiers() || isJavaIdentifier(name));
    }

    /**
     * checks if a property is a java identifier
     * 
     * @param s the property name
     * @return true if java identifier
     */
    private static boolean isJavaIdentifier(String s) {
        int n = s.length();
        if (n == 0) {
            return false;
        }
        if (!Character.isJavaIdentifierStart(s.charAt(0))) {
            return false;
        }
        for (int i = 1; i < n; i++) {
            if (!Character.isJavaIdentifierPart(s.charAt(i))) {
                return false;
            }
        }
        return true;
    }

    /**
     * @return Returns the needJavaIdentifiers.
     */
    public boolean isNeedJavaIdentifiers() {
        return this.needJavaIdentifiers;
    }

    /**
     * @param needJavaIdentifiers The needJavaIdentifiers to set.
     */
    public void setNeedJavaIdentifiers(boolean needJavaIdentifiers) {
        this.needJavaIdentifiers = needJavaIdentifiers;
    }
}
