/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.connectors.jms.marshaler;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.StringWriter;
import java.util.Map;

import javax.activation.DataHandler;
import javax.jms.Message;
import javax.jms.ObjectMessage;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.xml.stream.XMLStreamReader;
import javax.xml.transform.Source;
import javax.xml.transform.TransformerFactory;

import org.eclipse.ebpm.core.ExchangeUtils;
import org.eclipse.ebpm.messaging.api.Exchange;
import org.eclipse.ebpm.messaging.api.Pattern;
import org.eclipse.ebpm.soap.core.MessageImpl;
import org.eclipse.ebpm.soap.core.PhaseInterceptorChain;
import org.eclipse.ebpm.soap.interceptors.mime.AttachmentsInInterceptor;
import org.eclipse.ebpm.soap.interceptors.mime.AttachmentsOutInterceptor;
import org.eclipse.ebpm.soap.interceptors.xml.BodyOutInterceptor;
import org.eclipse.ebpm.soap.interceptors.xml.StaxInInterceptor;
import org.eclipse.ebpm.soap.interceptors.xml.StaxOutInterceptor;

public class DefaultConsumerMarshaler extends AbstractJmsMarshaler implements JmsConsumerMarshaler {
    
    private Pattern mep;
    private boolean rollbackOnError;
    private boolean rollbackOnErrorDefault;
    private boolean rollbackConfigured;

    public DefaultConsumerMarshaler() {
        this.mep = Pattern.InOnly;
    }
    
    public DefaultConsumerMarshaler(Pattern mep) {
        this.mep = mep;
    }

    /**
     * @return the mep
     */
    public Pattern getMep() {
        return mep;
    }

    /**
     * @param mep the mep to set
     */
    public void setMep(Pattern mep) {
        this.mep = mep;
    }

    public boolean isRollbackOnError() {
        return rollbackConfigured ? rollbackOnError : rollbackOnErrorDefault;
    }

    /**
     * @param rollbackOnError if exchange in errors should cause a rollback on the JMS side
     */
    public void setRollbackOnError(boolean rollbackOnError) {
        rollbackConfigured = true;
        this.rollbackOnError = rollbackOnError;
    }

    /**
     * This is called to set intelligent defaults if no
     * explicit rollbackOnError configuration is set.
     * If setRollbackOnError is explicitly set, it
     * will be used.
     *
     * @param rollbackDefault default rollbackOnError setting
     */
    public void setRollbackOnErrorDefault(boolean rollbackDefault) {
        this.rollbackOnErrorDefault = rollbackDefault;
    }

    public Exchange createExchange(Message message) throws Exception {
        Exchange exchange = ExchangeUtils.createExchange(mep);
        org.eclipse.ebpm.messaging.api.Message inMessage = exchange.getIn();
        populateMessage(message, inMessage);
        if (isCopyProperties()) {
            copyPropertiesFromJMS(message, inMessage);
        }
        return exchange;
    }

    public Message createOut(Exchange exchange, org.eclipse.ebpm.messaging.api.Message outMsg, Session session) throws Exception {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        PhaseInterceptorChain chain = new PhaseInterceptorChain();
        chain.add(new AttachmentsOutInterceptor());
        chain.add(new StaxOutInterceptor());
        chain.add(new BodyOutInterceptor());
        org.eclipse.ebpm.soap.api.Message msg = new MessageImpl();
        msg.setContent(Source.class, outMsg.getBody());
        msg.setContent(OutputStream.class, baos);
        for (String attId : outMsg.getAttachments().keySet()) {
            msg.getAttachments().put(attId, (DataHandler) outMsg.getAttachment(attId));
        }
        chain.doIntercept(msg);
        TextMessage text = session.createTextMessage(baos.toString());
        if (msg.get(org.eclipse.ebpm.soap.api.Message.CONTENT_TYPE) != null) {
            text.setStringProperty(CONTENT_TYPE_PROPERTY,
                                   (String) msg.get(org.eclipse.ebpm.soap.api.Message.CONTENT_TYPE));
        }
        if (isCopyProperties()) {
            copyPropertiesFromNM(outMsg, text);
        }
        return text;
    }

    public Message createFault(Exchange exchange, org.eclipse.ebpm.messaging.api.Message fault, Session session) throws Exception {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        PhaseInterceptorChain chain = new PhaseInterceptorChain();
        chain.add(new AttachmentsOutInterceptor());
        chain.add(new StaxOutInterceptor());
        chain.add(new BodyOutInterceptor());
        org.eclipse.ebpm.soap.api.Message msg = new MessageImpl();
        msg.setContent(Source.class, fault.getBody());
        msg.setContent(OutputStream.class, baos);
        for (String attId : fault.getAttachments().keySet()) {
            msg.getAttachments().put(attId, (DataHandler) fault.getAttachment(attId));
        }
        chain.doIntercept(msg);
        TextMessage text = session.createTextMessage(baos.toString());
        if (msg.get(org.eclipse.ebpm.soap.api.Message.CONTENT_TYPE) != null) {
            text.setStringProperty(CONTENT_TYPE_PROPERTY,
                                   (String) msg.get(org.eclipse.ebpm.soap.api.Message.CONTENT_TYPE));
        }
//        text.setBooleanProperty(FAULT_JMS_PROPERTY, true);
        if (isCopyProperties()) {
            copyPropertiesFromNM(fault, text);
        }
        return text;
    }

    public Message createError(Exchange exchange, Exception error, Session session) throws Exception {
        if (rollbackOnError) {
            throw error;
        } else {
            ObjectMessage message = session.createObjectMessage(error);
//            message.setBooleanProperty(ERROR_JMS_PROPERTY, true);
            return message;
        }
    }

    protected void populateMessage(Message message, org.eclipse.ebpm.messaging.api.Message normalizedMessage) throws Exception {
        if (message instanceof TextMessage) {
            PhaseInterceptorChain chain = new PhaseInterceptorChain();
            chain.add(new AttachmentsInInterceptor());
            chain.add(new StaxInInterceptor());
            org.eclipse.ebpm.soap.api.Message msg = new MessageImpl();
            msg.setContent(InputStream.class, new ByteArrayInputStream(((TextMessage) message).getText().getBytes()));
            if (message.propertyExists(CONTENT_TYPE_PROPERTY)) {
                msg.put(org.eclipse.ebpm.soap.api.Message.CONTENT_TYPE, message.getStringProperty(CONTENT_TYPE_PROPERTY));
            }
            chain.doIntercept(msg);
            XMLStreamReader xmlReader = msg.getContent(XMLStreamReader.class);            
            javax.xml.transform.stax.StAXSource src = new javax.xml.transform.stax.StAXSource(xmlReader);
            
            javax.xml.transform.TransformerFactory tfactory = TransformerFactory.newInstance();
            javax.xml.transform.Transformer xform = tfactory.newTransformer();
            java.io.StringWriter writer = new StringWriter();
            javax.xml.transform.Result result = new javax.xml.transform.stream.StreamResult(writer);
            xform.transform(src, result);

            normalizedMessage.setBody(writer.toString());            
            
            for (Map.Entry<String, DataHandler> attachment : msg.getAttachments().entrySet()) {
                normalizedMessage.addAttachment(attachment.getKey(), attachment.getValue());
            }
        } else {
            throw new UnsupportedOperationException("JMS message is not a TextMessage");
        }
    }

}
