/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/

package org.eclipse.ebpm.connectors.mail.marshaler;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.mail.internet.MimeMessage;

import org.eclipse.ebpm.messaging.api.Exchange;
import org.eclipse.ebpm.messaging.api.Message;

/**
 * this is the abstract super class of all marshalers which want to convert
 * between normalized messages and mail messages and vice versa.
 * 
 * @author lhein  for original sources
 * @author Andrea Zoppello for further modifications
 */
public abstract class AbstractMailMarshaler {
    /**
     * ------------------------------------------------------------------------
     * the mail tags are used for parsing a mime mail for specific tags
     * ------------------------------------------------------------------------
     */

    /**
     * this is the tag for the "To" field of the email
     */
    public static final String MAIL_TAG_TO = "To";

    /**
     * this is the tag for the "Cc" field of the email
     */
    public static final String MAIL_TAG_CC = "Cc";

    /**
     * this is the tag for the "Bcc" field of the email
     */
    public static final String MAIL_TAG_BCC = "Bcc";

    /**
     * this is the tag for the "From" field of the email
     */
    public static final String MAIL_TAG_FROM = "From";

    /**
     * this is the tag for the "Subject" field of the email
     */
    public static final String MAIL_TAG_SUBJECT = "Subject";

    /**
     * this is the tag for the "Reply-To" field of the email
     */
    public static final String MAIL_TAG_REPLYTO = "Reply-To";

    /**
     * this is the tag for the "Date" field of the email
     */
    public static final String MAIL_TAG_SENTDATE = "Date";

    /**
     * ------------------------------------------------------------------------
     * the email constants are used for transporting specific information in 
     * message properties
     * ------------------------------------------------------------------------
     */

    /**
     * this tag is used in normalized messages to represent the address the 
     * mail was sent to OR the address this exchange should be mailed to
     */
    public static final String MSG_TAG_TO = "org.spagic3.mail.to";

    /**
     * this tag is used in normalized messages to represent the CC address the 
     * mail was sent to OR the CC address this exchange should be mailed to
     */
    public static final String MSG_TAG_CC = "org.spagic3.mail.cc";

    /**
     * this tag is used in normalized messages to represent the BCC address 
     * this exchange should be mailed to
     */
    public static final String MSG_TAG_BCC = "org.spagic3.mail.bcc";

    /**
     * this tag is used in normalized messages to represent the address the 
     * mail was received from OR the address this exchange should be sent from
     */
    public static final String MSG_TAG_FROM = "org.spagic3.mail.from";

    /**
     * this tag is used in normalized messages to represent the text portion 
     * of the mail received OR the text portion of the mail to be sent
     * if both html and text properties are set, then it will be a multipart
     * mail message containing both
     */
    public static final String MSG_TAG_TEXT = "org.spagic3.mail.text";

    /**
     * this tag is used in normalized messages to represent the html portion 
     * of the mail received OR the html portion of the mail to be sent
     * if both html and text properties are set, then it will be a multipart
     * mail message containing both 
     */
    public static final String MSG_TAG_HTML = "org.spagic3.mail.html";

    /**
     * this tag is used in normalized messages to represent the subject of the 
     * mail received OR the subject of the mail to be sent
     */
    public static final String MSG_TAG_SUBJECT = "org.spagic3.mail.subject";

    /**
     * this tag is used in normalized messages to represent the reply-to address
     * of the mail received OR the reply-to address of the mail to be sent
     */
    public static final String MSG_TAG_REPLYTO = "org.spagic3.mail.replyto";

    /**
     * this tag is used in normalized messages to represent the date when  
     * the mail was sent
     */
    public static final String MSG_TAG_SENTDATE = "org.spagic3.mail.sentdate";

    /**
     * this tag is used in normalized messages to represent the alternative
     * content of a multipart/related message type
     */
    public static final String MSG_TAG_ALTERNATIVE_CONTENT = "org.spagic3.mail.alternativecontent";

    /**
     * the default sender address for outgoing mails
     */
    public static final String DEFAULT_SENDER = "no-reply@localhost";

    /**
     * the dummy subject - set if no subject was provided
     */
    public static final String DUMMY_SUBJECT = "no subject";
    
    /**
     * the dummy content - set if no content was provided
     */
    public static final String DUMMY_CONTENT = "no content";
    
    private Map< String, List<File> > temporaryFilesMap = Collections.synchronizedMap(new HashMap<String, List<File> >());
    
    /**
     * This method is used to convert a mime mail message received via an 
     * email server into a normalized message which will be sent to the bus.
     * If you want to specify your own conversion behaviour you have to override
     * this method with your own logic.
     * 
     * @param exchange  the message exchange that will be sent to the bus
     * @param nmsg      the normalized in-message to be filled by this method
     * @param mailMsg   the mail message that was received via mail server
     * @throws javax.mail.MessagingException on any conversion errors
     */
    public abstract void convertMailToNormalizedMessage(Exchange exchange, Message msg,
                                          MimeMessage mailMsg) throws javax.mail.MessagingException;

    /**
     * This method is used to convert a normalized message from the bus into a
     * mime mail message to be sent to a mail server.
     * If you want to specify your own conversion behaviour you have to override
     * this method with your own logic. 
     * 
     * @param mimeMessage               the mime mail message to be filled by this method
     * @param exchange                  the message exchange from  bus
     * @param nmsg                      the normalized message to transform to mail message
     * @param configuredSender          the sender configured in the xbean
     * @param configuredReceiver        the receiver configured in the xbean
     * @throws javax.mail.MessagingException on conversion errors
     */
    public abstract void convertNormalizedMessageToMail(MimeMessage mimeMessage, Exchange exchange, Message msg, 
    		String configuredSender, String configuredReceiver) throws javax.mail.MessagingException;

    /**
     * returns the default sender for outgoing mails
     * Override this method to deliver your own default sender!
     * 
     * @return  the default sender address as string
     */
    public String getDefaultSenderForOutgoingMails() {
        return AbstractMailMarshaler.DEFAULT_SENDER;
    }

    /**
     * This helper method extracts all attachments from the normalized message 
     * into a map of attachments which may be used for filling the attachments
     * of an outgoing mail
     * 
     * @param normalizedMessage     the normalized message with attachments
     * @return  a map of attachments
     */
    protected final Map<String, DataSource> getAttachmentsMapFromNormalizedMessage(Message message) {
        // get attachment from Normalize Message (used for sending)
        Map<String, DataSource> attachments = new HashMap<String, DataSource>();
        String oneAttachmentName = "";
        Map att = message.getAttachments();
        Set attNames = att.keySet();
//        Set attNames = message.getAttachmentNames();
        Iterator itAttNames = attNames.iterator();
        while (itAttNames.hasNext()) {
            oneAttachmentName = (String)itAttNames.next();
//            DataSource oneAttchmentInputString = normalizedMessage.getAttachment(oneAttachmentName).getDataSource();
            DataSource oneAttchmentInputString = ((DataHandler)message.getAttachment(oneAttachmentName)).getDataSource();
            attachments.put(oneAttachmentName, oneAttchmentInputString);
        }

        return attachments;
    }
    
    /**
     * adds a temporary file resource to the list of to be cleaned up resources
     * 
     * @param id        the id of the message exchange
     * @param tmpFile   the temp resource
     */
    protected final void addTemporaryResource(String id, File tmpFile) { 
        if (!this.temporaryFilesMap.containsKey(id)) {
            this.temporaryFilesMap.put(id, new ArrayList<File>()); 
        }
        this.temporaryFilesMap.get(id).add(tmpFile); 
    }

    /**
     * deletes all temporary resources of a given exchange id
     * 
     * @param id        the exchange id
     */
    public final void cleanUpResources(String id) {
        List<File> list = this.temporaryFilesMap.get(id);
        if (list != null) {
            for (File f : list) { 
                f.delete(); 
            }
            list.clear();
            this.temporaryFilesMap.remove(id);
        }
    }
}
