/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.core;

import org.eclipse.ebpm.core.routing.IMessageRouter;
import org.eclipse.ebpm.messaging.api.Exchange;


/**
 * Interface <code>ISpagicService</code> Contract that all services
 * and connectors in Spagic must respect.
 *
 * @author <a href="mailto:Gianfranco.Boccalon@eng.it">Gianfranco Boccalon</a>
 * @version $Revision$
 */
public interface ISpagicService {

	/**
	 * <code>getSpagicId</code> returns the id of the connector/service
	 *
	 * @return a <code>String</code> the id of the connector/service
	 */
	public String getSpagicId();

	/**
	 * This is the method that is invoked when the service
	 * received an Exchange from the NMR
	 * 
	 * @param exchange an <code>Exchange</code> the exchange received
	 * @exception Exception
	 */
	public void process(Exchange exchange); 

	/**
	 * Methods to bind with the Message Router Service
	 * @param router
	 */
	public void setMessageRouter(IMessageRouter router);

	/**
	 * <code>unsetMessageRouter</code> Methods to unbind from the
	 * Message Router Service
	 *
	 * @param router an <code>IMessageRouter</code>, the router to
	 * undbind from
	 */
	public void unsetMessageRouter(IMessageRouter router);

	/**
	 * <code>getMessageRouter</code> returns the current message router
	 *
	 * @return an <code>IMessageRouter</code> the current message router
	 */
	public IMessageRouter getMessageRouter();

	/**
	 * <code>init</code> inits the connector/service
	 *
	 */
	public void init();

	/**
	 * <code>beforeDestroy</code> last method invoked before
	 * connector/service destruction. Final cleanup of resources that
	 * must be released manually goes there.
	 *
	 */
	public void beforeDestroy();

	/**
	 * Implementors of this interface must provide a behaviour that mantains
	 * consistence with the isStarted() and isStopped() predicates. 
	 * After calling this method should hold that isStarted is true 
	 * and isStopped is false.
	 * @throws Exception 
	 */
	public void start() throws Exception;

	/**
	 * Implementors of this interface must provide a behaviour that mantains
	 * consistence with the isStarted() and isStopped() predicates.
	 * After calling this method should hold that isStopped is true 
	 * and isStarted is false.
	 * @throws Exception 
	 */
	public void stop() throws Exception;

	/**
	 * Implementors of this interface must provide a behaviour that mantains
	 * consistence with the start() and stop() methods. After calling
	 * start() this method predicate will be true and must become false after a
	 * call to stop(). It must hold always that isStarted() == ! isStopped().
	 * @return a <code>boolean</code> true if the connector/service is started
	 * @exception Exception because some implementors require it.
	 */
	public boolean isStarted() throws Exception;

	/**
	 * Implementors of this interface must provide a behaviour that mantains
	 * consistence with the start() and stop() methods. After calling
	 * stop() this predicate will be true and must become false after a
	 * call to start(). It must hold always that isStarted() == ! isStopped().
	 * @return a <code>boolean</code> false if the connector/service is started
	 * @exception Exception because some implementors require it.
	 */
	public boolean isStopped() throws Exception;

}
