/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.databasemanager;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.NoResultException;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.sql.DataSource;

import org.eclipse.ebpm.constants.SpagicConstants;
import org.eclipse.ebpm.metadb.constants.ProcessExecutionStateConstants;
import org.eclipse.ebpm.metadb.constants.PropertyServiceInstanceConstants;
import org.eclipse.ebpm.metadb.constants.ServiceInstanceStateConstants;
import org.eclipse.ebpm.metadb.constants.TypeServiceConstants;
import org.eclipse.ebpm.metadb.model.Component;
import org.eclipse.ebpm.metadb.model.Property;
import org.eclipse.ebpm.metadb.model.PropertyServiceInstance;
import org.eclipse.ebpm.metadb.model.Service;
import org.eclipse.ebpm.metadb.model.ServiceInstance;
import org.eclipse.ebpm.metadb.model.TypeService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class DatabaseService implements IDatabaseManager {
	
	protected Logger log = LoggerFactory.getLogger(DatabaseService.class);

	public final static String COMPONENT_ID = "idComponent".intern();
	public final static String COMPONENT_NAME = "name".intern();
	
	public final static String SERVICE_ID = "idService".intern();
	public final static String SERVICE_VERSION = "serviceVersion".intern();
	
	public final static String SERVICE_INSTANCE_ID = "idServiceInstance".intern();
	public final static String SERVICE_INSTANCE_SERVICE_ID = "id_service".intern();
	public final static String SERVICE_INSTANCE_MESSAGE_ID = "messageId".intern();
	public final static String SERVICE_INSTANCE_CORRELATION_ID = "correlationId".intern();
	
	public static final String NON_JTA_DATASOURCE = "javax.persistence.nonJtaDataSource";
	public static final String PERSISTENCE_UNIT = "org.eclipse.ebpm.metadb.model";
	
	private Map<String, DataSource> properties = null;
	
	private EntityManagerFactory emf = null;
	private EntityManager em = null;
	
	public void bindMetaDB(javax.sql.DataSource ds){
		log.info("DatabaseService: Metadb Datasource has been bound");
		this.properties = new HashMap<String, DataSource>();
	    properties.put(NON_JTA_DATASOURCE, ds);
	    emf = Persistence.createEntityManagerFactory(PERSISTENCE_UNIT, properties);
		em = emf.createEntityManager();
	}
	
	public void unbindMetaDB(javax.sql.DataSource ds){
		log.info("DatabaseService: Metadb Datasource has been unbound");
	}
	
	@Override
	public Component getComponentByName(String componentName) {
		
		try{
			Query query = em.createQuery("select c from Component c where c.name = :name");
			query.setParameter(COMPONENT_NAME, componentName);
			return (Component)query.getSingleResult();
		}catch (NoResultException e) {
			return null;
		}
	}

	@Override
	public Service getServiceById(String serviceId) {
		try{
			//eseguo la select di un singolo record
			return em.find(Service.class, serviceId);
		}catch (NoResultException e) {
			return null;
		}
	}
	
//	@Override
//	public Service getServiceByIdAndVersion(String serviceId, String version) {
//		
//		try{
//			Query query = em.createQuery("select from service where serviceId = :serviceId and version = :version");
//			query.setParameter(SERVICE_ID, serviceId);
//			query.setParameter(SERVICE_VERSION, version);
//			return (Service)query.getSingleResult();
//		}catch (Throwable e) {
//			throw new RuntimeException("Component retrieve failed", e);
//		}
//	}
	
	@Override
	public ServiceInstance getServiceInstance(Long serviceInstanceId) {
		
		try{
			return em.find(ServiceInstance.class, serviceInstanceId);
		}catch (NoResultException e) {
			return null;
		}
	}

	@Override
	public ServiceInstance getServiceInstanceByMessageId(String exchangeID) {
		
		try{
			Query query = em.createQuery("select si from ServiceInstance si where si.messageId = :messageId");
			query.setParameter(SERVICE_INSTANCE_MESSAGE_ID, exchangeID);
			return (ServiceInstance)query.getSingleResult();
		}catch (NoResultException e) {
			return null;
		}

	}
	
	
	@Override
	public ServiceInstance getServiceInstanceByCorrelationId(String serviceId, String correlationID) {
		
		try{
			Query query = em.createQuery("select si from ServiceInstance si where si.service.idService = :id_service and si.correlationId = :correlationId");
			query.setParameter(SERVICE_INSTANCE_SERVICE_ID, serviceId);
			query.setParameter(SERVICE_INSTANCE_CORRELATION_ID, correlationID);
			return (ServiceInstance)query.getSingleResult();
		}catch (NoResultException e) {
			return null;
		}
	}

	@Override
	public ServiceInstance createServiceInstance(String serviceId,String exchangeID, String correlationId, 
		   ServiceInstance targetServiceInstance, String request, String response) {
		
		EntityTransaction tx = null;		
		ServiceInstance serviceInstance = null;

		try {			
			serviceInstance = new ServiceInstance();
			serviceInstance.setMessageId(exchangeID);
			serviceInstance.setCorrelationId(correlationId);	
			serviceInstance.setRequest(request);
			serviceInstance.setResponse(response);
			serviceInstance.setService(getServiceById(serviceId));
			serviceInstance.setTargetServiceInstance(targetServiceInstance);
			serviceInstance.setState(ServiceInstanceStateConstants.SERVICE_ACTIVE);
			serviceInstance.setStartdate(new Date());
			
			tx = em.getTransaction();
			tx.begin();
			em.persist(serviceInstance);
			tx.commit();			
		} catch (Throwable ex) {
			log.error("Error creating service instance: " + serviceId + "," + correlationId, ex);
			if (tx != null && tx.isActive()) {
				tx.rollback();
			}
		}
		
		return serviceInstance;
	}

	@Override
	public void updateServiceInstance(ServiceInstance serviceInstance) {
		
		EntityTransaction tx = null;
		try {	
			tx = em.getTransaction();
			tx.begin();
			serviceInstance = em.merge(serviceInstance);
			tx.commit();
		}catch (Throwable e) {
			if (tx != null && tx.isActive()) {
				tx.rollback();
			}
			log.error("Error updating service instance: " + serviceInstance.getIdServiceInstance(), e);
			log.error("Try to create a new ServiceInstance.");
			try {
				tx = em.getTransaction();
				tx.begin();
				em.persist(serviceInstance);
				tx.commit();
			}catch (Throwable ex) {
				if (tx != null && tx.isActive()) {
					tx.rollback();
				}
				throw new RuntimeException("ServiceInstance updateServiceInstance failed: "+ serviceInstance.getIdServiceInstance());
			}
		}
	}

	@Override
	public Service registerService(String serviceId, String componentName, Map<String, String> properties) {
		
		Service service = getServiceById(serviceId);
		EntityTransaction tx = null;
		if (service != null) {
			// Service already registered: check if we must update it
			return service;				
		}
		try {
			Component component = getComponentByName(componentName);
			if (component != null){
				tx = em.getTransaction();
				tx.begin();

				// Save the service
				service = new Service(serviceId, component, ProcessExecutionStateConstants.NORMAL_EXECUTION.intValue(), true, new Date(), null, null, null);
				em.persist(service);
				
				
				List<Service> referencedServiceAsList = new ArrayList<Service>();
				referencedServiceAsList.add(service);
				
				//List<Property> propertiesList = new ArrayList<Property>(properties.size());
	
				String propValue = null;
				Property metaDBProperty = null;
				for (String propKey : properties.keySet()) {
					
					propValue = properties.get(propKey);
					metaDBProperty = new Property();
					metaDBProperty.setCode(propKey);
					metaDBProperty.setValue(propValue);
					metaDBProperty.setServices(referencedServiceAsList);
					em.persist(metaDBProperty);
					//propertiesList.add(metaDBProperty);
				}
	
//				service = new Service(serviceId, component, ProcessExecutionStateConstants.NORMAL_EXECUTION.intValue(), true, new Date(), null, propertiesList, null);			
//				em.persist(service);
				tx.commit();
			}else{
				log.warn("Component with name [" + componentName +"] is not registered ");
			}
		}catch (Throwable ex) {
			log.error("Error registering component: " + componentName, ex);
			if (tx != null && tx.isActive()) {
				tx.rollback();
			}
		} 
		return service;
	}

	@Override
	public Component registerComponent(String componentName, String componentType, Map<String, String> properties) {

		Component component = null;
		EntityTransaction tx = null;
		try {
			// Check if the service is already registered
			component = getComponentByName(componentName);
			if (component != null) {
				// Component already registered: check if we must update it
				return component;				
			}
			
			tx = em.getTransaction();
			tx.begin();

			TypeService typeService = null;
			if (componentType.equals(SpagicConstants.SERVICE_TYPE_CONNECTOR)) {
				typeService = em.find(TypeService.class, TypeServiceConstants.OSGI_CONNECTOR);
			} else if (componentType.equals(SpagicConstants.SERVICE_TYPE_SERVICE)) {
				typeService = em.find(TypeService.class, TypeServiceConstants.OSGI_SERVICE);
			}
			
			component = new Component(typeService, componentName, ProcessExecutionStateConstants.NORMAL_EXECUTION.intValue(), null, null);
			em.persist(component);
			
			List<Component> referencedComponentAsList = new ArrayList<Component>();
			referencedComponentAsList.add(component);
			
			List<Property> propertiesSet = new ArrayList<Property>(properties.size());
			String propValue = null;
			Property metaDBProperty = null;
			for (String propKey : properties.keySet()) {
				
				propValue = properties.get(propKey);
				metaDBProperty = new Property();	
				metaDBProperty.setCode(propKey);
				metaDBProperty.setValue(propValue);
				metaDBProperty.setComponents(referencedComponentAsList);
				em.persist(metaDBProperty);			
				propertiesSet.add(metaDBProperty);
			}
			
			tx.commit();
			return component;
		} catch (Throwable ex) {
			log.error("Error registering component: " + componentName, ex);
			if (tx != null && tx.isActive()) {
				tx.rollback();
			}
		} 
		return component;
	}
	
	@Override
	public void relateProcessToServiceInstance(String exchangeId, Long processId) {

		EntityTransaction tx = null;
		ServiceInstance serviceInstance = getServiceInstanceByMessageId(exchangeId);				
		PropertyServiceInstance psi = new PropertyServiceInstance(serviceInstance, PropertyServiceInstanceConstants.PROCESS_ID, processId.toString());
		try{
			tx = em.getTransaction();
			tx.begin();
			em.persist(psi);
			tx.commit();
		}catch (Throwable e) {
			log.error("Error relating process to service instance: " + exchangeId + "," + processId, e);
			if (tx != null && tx.isActive()) {
				tx.rollback();
			}
		}	
		
	}
	

}
