/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.dynapp;

import java.io.File;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.ConcurrentHashMap;

import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.eclipse.ebpm.constants.SpagicConstants;
import org.osgi.framework.BundleContext;
import org.osgi.service.component.ComponentContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class DynamicApplicationWatcherService {
	
	protected Logger logger = LoggerFactory.getLogger(DynamicApplicationWatcherService.class); 
	
	protected File spagicApplicationsFile = null;
	protected Timer timer = null;
	
	
	protected ConcurrentHashMap<String, ApplicationHandler> applications = new ConcurrentHashMap<String, ApplicationHandler>();
	
	protected Map<String, ApplicationHandler> loadApplications(BundleContext context) {
		
		String spagicHome = System.getProperty("spagic.home");
		Map<String, ApplicationHandler> apps = new HashMap<String, ApplicationHandler>();
		try{
	 	
		SAXReader reader = new SAXReader();
		Document doc = reader.read(spagicApplicationsFile);
			
			
		Element rootAppEl = doc.getRootElement();
			
		List<Element> applicationsElements = rootAppEl.elements("spagic-application");
		
		for (Element appEl: applicationsElements) {
			
			// Verify if the application was already there
			ApplicationHandler appHandler = new ApplicationHandler( context,spagicHome,appEl);
			apps.put(appHandler.getApplicationId(), appHandler);
			
		}
		}catch (Exception e) {
			logger.error(e.getMessage(),e);
		}finally{
			return apps;
		}
		
	}
	protected void activate(ComponentContext componentContext){
		
		logger.debug("DynamicApplicationWatcherService -> activate ");
		String spagicHome = System.getProperty("spagic.home");
		
		if (spagicHome == null){
			logger.warn(" Spagic HOME NOT SETTED ");
			return;
		}
			
		final String spagicApplicationFileName =  spagicHome+ File.separator + SpagicConstants.SPAGIC_APPLICATIONS_FILE_NAME;
		logger.debug("DynamicApplicationWatcherService ->  Spagic Applications File Name ["+spagicApplicationFileName+"]");
		
		this.spagicApplicationsFile = new File(spagicApplicationFileName);
		
		//
		//
		// loadApplications 
		// <spagic-applications>
		//		<spagic-application id="MyApp" name="My Application" path="AAAA" type="relative(absolute)" />"
		//      <spagic-application id="MyApp2" name="My Application" path="AAAA" type="relative(absolute)" />"
		//			
		//
		//
		//
		//

		
		final BundleContext bundleContext = componentContext.getBundleContext();
		
		logger.debug("DynamicApplicationWatcherService ->  Initial Load of Spagic Applications from ["+spagicApplicationFileName+"]");
		Map<String, ApplicationHandler> newApps = loadApplications(bundleContext);
		
		for ( String key : newApps.keySet()){
			logger.debug("DynamicApplicationWatcherService ->  Initial Load :: Found Spagic Application ["+key+"]");
			applications.put(key, newApps.get(key));
			logger.debug("DynamicApplicationWatcherService ->   Initial Load :: Start Spagic Application ["+key+"]");
			applications.get(key).start();
			logger.debug("DynamicApplicationWatcherService ->  Initial Load :: Started Spagic Application ["+key+"]");
		}
		
		TimerTask task = new FileWatcher( this.spagicApplicationsFile ) {
		      protected void onChange( File file ) {
		    	  logger.debug("DynamicApplicationWatcherService ->  Spagic App Thread ["+spagicApplicationFileName+"] Changed Reload Applications");
		    	  Map<String, ApplicationHandler> newApps = loadApplications(bundleContext);
		    	  
		    	  logger.debug("DynamicApplicationWatcherService ->  Spagic App Thread ["+spagicApplicationFileName+"] Applications Reloaded");
		    	 
		    	  
		    	  for ( String key : applications.keySet()){
		    		  if (!(newApps.containsKey(key))){
		    			 logger.debug("DynamicApplicationWatcherService ->  Application ["+key+"] HAVE BEEN REMOVED from new list");
		    			 applications.get(key).stop();
		    			 applications.remove(key);
		    		  }  
		    	  }
		    	  
		    	  // New Applications
		    	  for ( String key : newApps.keySet()){
		    		  if (!(applications.containsKey(key))){
		    			  logger.debug("DynamicApplicationWatcherService ->  Application ["+key+"] is NEW APPLICATION Initialize and start it ");
		    			  ApplicationHandler appH = newApps.get(key);
		    			  applications.put(key, appH);
		    			  appH.start();
		    		  }
		    	  }
		    	  
		      }
		};

		logger.debug("DynamicApplicationWatcherService ->  Scheduling Thread For Polling Spagic-Applications");
		timer = new Timer();
		timer.schedule( task , new Date(), 5000 );
		
		logger.debug("DynamicApplicationWatcherService ->  Thread Scheduled");

		
		
		
		
	}
	
	protected void deactivate(ComponentContext componentContext){
		
		timer.cancel();
		
		for ( String key : applications.keySet()){
			applications.get(key).stop();
		}
		
		applications.clear();
	}
	
}
