/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/

package org.eclipse.ebpm.messaging.api;

import java.io.Serializable;
import java.util.Map;

import javax.xml.namespace.QName;


public interface Exchange extends Serializable {

    /**
     * The unique id of this exchange
     * @return
     */
    String getId();

    /**
     * The exchange pattern
     * @return
     */
    Pattern getPattern();

   
    
    /**
     * The status of the exchange
     * @return
     */
    Status getStatus();

    /**
     * Set the status of the exchange
     *
     * @param status the new status
     */
    void setStatus(Status status);

    

    /**
     * The service operation of this exchange
     *
     * @return  the operation
     */
    QName getOperation();

    /**
     * The service operation of this exchange
     *
     * @param operation the operation
     */
    void setOperation(QName operation);

    /**
     * Get a given property by its name.
     *
     * @param name the name of the property to retrieve
     * @return the value of the property or <code>null</code> if none has been set
     */
    Object getProperty(String name);

   

   

    /**
     * Return all the properties associated with this exchange
     *
     * @return all the properties
     */
    Map<String, Object> getProperties();

    /**
     * Set a property on this exchange.
     * Giving <code>null</code> will actually remove the property for the list.
     *
     * @param name the name of the property
     * @param value the value for this property or <code>null</code>
     */
    void setProperty(String name, Object value);

   
    /**
     * Remove the given property and returns its value.
     *
     * @param name the name of the property
     * @return the previous value
     */
    Object removeProperty(String name);

    

    /**
     * Set all the properties
     *
     * @param properties the new properties
     */
    void setProperties(Map<String, Object> properties);

    /**
     * Obtains the input message, lazily creating one if none
     * has been associated with this exchange. If you want to inspect this property
     * but not force lazy creation then invoke the {@link #getIn(boolean)}
     * method passing in false
     *
     * @return the input message
     */
    Message getIn();

    /**
     * Returns the inbound message, optionally creating one if one has not already
     * been associated with this exchange.
     *
     * @param lazyCreate <code>true</code> if the message should be created
     * @return the input message
     */
    Message getIn(boolean lazyCreate);

    /**
     * Set the inbound message.
     *
     * @param message the new inbound message
     */
    void setIn(Message message);
    
    /**
     * Obtains the outbound message, lazily creating one if none
     * has been associated with this exchange and if this exchange
     * supports an out message. If you want to inspect this property
     * but not force lazy creation then invoke the {@link #getOut(boolean)}
     * method passing in false
     *
     * @return the output message
     */
    Message getOut();

    /**
     * Returns the outbound message, optionally creating one if one has not already
     * been associated with this exchange
     *
     * @return the out message
     */
    Message getOut(boolean lazyCreate);

    /**
     * Set the outbound message.
     *
     * @param message the new outbound message
     */
    void setOut(Message message);

    /**
     * Obtains the fault message, lazily creating one if none
     * has been associated with this exchange and if this exchange
     * supports a faut message. If you want to inspect this property
     * but not force lazy creation then invoke the {@link #getFault(boolean)}
     * method passing in false
     *
     * @return the fault message
     */
    Message getFault();

    /**
     * Returns the fault message, optionally creating one if one has not already
     * been associated with this exchange
     *
     * @return the fault message
     */
    Message getFault(boolean lazyCreate);

    /**
     * Set the fault message.
     *
     * @param message the new fault message
     */
    void setFault(Message message);

    /**
     * Obtains the given message, lazily creating one if none
     * has been associated with this exchange and if this exchange
     * supports a faut message. If you want to inspect this property
     * but not force lazy creation then invoke the {@link #getMessage(Type, boolean)}
     * method passing in false
     *
     * @param type the type of message to retrieve
     * @return the message or <code>null</code> if
     *         this pattern does not support this type of message
     */
    Message getMessage(Type type);

    /**
     * Returns the message of the given type, optionally creating one if one has not already
     * been associated with this exchange
     *
     * @param type the type of message to retrieve
     * @return the given message
     */
    Message getMessage(Type type, boolean lazyCreate);

    /**
     * Set the message.
     *
     * @param type the type of the message to set
     * @param message the new inbound message
     */
    void setMessage(Type type, Message message);

    /**
     * Obtains the error of this exchange
     *
     * @return the exception that caused the exchange to fail
     */
    Exception getError();

    /**
     * Set the error on this exchange
     *
     * @param error the exception that caused the exchange to fail
     */
    void setError(Exception error);

    /**
     * Copy the given exchange to this one
     * @param exchange the exchange to copy from
     */
    void copyFrom(Exchange exchange);

    /**
     * Duplicates this exchange and returns a new copy
     *
     * @return a copy of this exchange
     */
    Exchange copy();

   

    /**
     * Returns a string representation of this exchange.
     * The <code>displayContent</code> property can be specified to
     * control wether the content should be displayed or not.
     * If set to <code>true</code>, the content and attachments will
     * be converted into re-readable objects (for java.io.InputStream
     * and javax.xml.transform.Source objects) and printed.
     *
     * @param displayContent if the content and attachments of the exchange
     *                       should be transformed if needed and displayed
     *                       or left unmodified
     * @return the string representation
     */
    String display(boolean displayContent);
    
    

}
