/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/

package org.eclipse.ebpm.messaging.api;

import java.io.Serializable;
import java.util.Map;

import javax.security.auth.Subject;
import javax.xml.transform.Source;


public interface Message extends Serializable {

    /**
     * Get a header on this message.
     *
     * @param name the name of the header
     * @return the value of the header of <code>null</code> if none has been set
     */
    Object getHeader(String name);
    

    /**
     * Set a header for this message
     * @param name the name of the header
     * @param value the value of the header
     */
    void setHeader(String name, Object value);

   

    /**
     * Remove the given header and returns its value.
     *
     * @param name the name of the header
     * @return the previous value
     */
    Object removeHeader(String name);

    

    /**
     * Get a map of all the headers for this message
     *
     * @return a map of headers
     */
    Map<String, Object> getHeaders();

    /**
     * Set all the headers
     *
     * @param headers the new map of headers
     */
    void setHeaders(Map<String, Object> headers);

    /**
     * Retrieve an attachment given its id.
     *
     * @param id the id of the attachment to retrieve
     * @return the attachement or <code>null</code> if none exists
     */
    Object getAttachment(String id);

    /**
     * Add an attachment to this message
     *
     * @param id the id of the attachment
     * @param value the attachment to add
     */
    void addAttachment(String id, Object value);

    /**
     * Remove an attachment on this message
     *
     * @param id the id of the attachment to remove
     */
    void removeAttachment(String id);

    /**
     * Retrieve a map of all attachments
     *
     * @return the map of attachments
     */
    Map<String, Object> getAttachments();

    /**
     * Returns the body of the message in its default format.
     *
     */
    Source getBody();

    

    /**
     * Set the body of the message.
     *
     * @param body the body of the message
     */
    void setBody(Source body);
    
    void setBody(String xml);


   

    /**
     * Get the mime content type describing the content of the message
     *
     * @return the mime content type
     */
    String getContentType();

    /**
     * Set the mime content type describing the content of the message
     *
     * @param type the mime content type
     */
    void setContentType(String type);

    /**
     * Get the encoding of the message
     *
     * @return the encoding
     */
    String getContentEncoding();

    /**
     * Set the encoding of the message
     *
     * @param encoding the encoding
     */
    void setContentEncoding(String encoding);

    /**
     * Get the security subject for this message
     *
     * @return the security subject
     */
    Subject getSecuritySubject();

    /**
     * Set the security subject for this message
     *
     * @param securitySubject the security subject
     */
    void setSecuritySubject(Subject securitySubject);

    /**
     * Copies the contents of the other message into this message
     * 
     * @param msg the message to copy from
     */
    void copyFrom(Message msg);

    /**
     * Creates a copy of this message so that it can
     * be used and possibly modified further in another exchange
     * 
     * @return a new message instance copied from this message
     */
    Message copy();

    String getBodyText();
    
    void setBodyText(String xml);
    

    /**
     * Returns a string representation of this message.
     * The <code>displayContent</code> property can be specified to
     * control wether the content should be displayed or not.
     * If set to <code>true</code>, the content and attachments will
     * be converted into re-readable objects (for java.io.InputStream
     * and javax.xml.transform.Source objects) and printed.
     *
     * @param displayContent if the content and attachments of the exchange
     *                       should be transformed if needed and displayed
     *                       or left unmodified
     * @return the string representation
     */
    String display(boolean displayContent);

}
