/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
/* This file contains code modified from Apache code */
package org.eclipse.ebpm.soap.util.stax;

import java.util.ArrayList;
import java.util.List;

import javax.xml.namespace.NamespaceContext;
import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;
import javax.xml.stream.util.StreamReaderDelegate;

public class FragmentStreamReader extends StreamReaderDelegate implements XMLStreamReader {

	protected static final int STATE_START_DOC = 0;
	protected static final int STATE_FIRST_ELEM = 1;
	protected static final int STATE_FIRST_RUN = 2;
	protected static final int STATE_RUN = 3;
	protected static final int STATE_END_DOC = 4;
	
	protected int depth;
	protected int state = STATE_START_DOC;
	protected int event = START_DOCUMENT;
    protected List<String> rootPrefixes;
	
    public FragmentStreamReader(XMLStreamReader parent) {
        super(parent);
        NamespaceContext ctx = getParent().getNamespaceContext();
        if (ctx instanceof ExtendedNamespaceContext) {
            rootPrefixes = new ArrayList<String>(((ExtendedNamespaceContext) ctx).getPrefixes());
        }
    }

	public int getEventType() {
		return event;
	}

	public int next() throws XMLStreamException {
		switch (state) {
		case STATE_START_DOC:
			state = STATE_FIRST_ELEM;
			event = START_DOCUMENT;
			break;
		case STATE_FIRST_ELEM:
			state = STATE_FIRST_RUN;
			depth++;
			event = START_ELEMENT;
			break;
		case STATE_FIRST_RUN:
			state = STATE_RUN;
			// do not break
		case STATE_RUN:
			event = getParent().next();
			if (event == START_ELEMENT) {
				depth++;
			} else if (event == END_ELEMENT) {
				depth--;
				if (depth == 0) {
					state = STATE_END_DOC;
				}
			}
			break;
		case STATE_END_DOC:
			event = END_DOCUMENT;
			break;
		default:
			throw new IllegalStateException();
		}
		return event;
	}

    public int nextTag() throws XMLStreamException {
        int eventType = next();
        while((eventType == XMLStreamConstants.CHARACTERS && isWhiteSpace()) // skip whitespace
            || (eventType == XMLStreamConstants.CDATA && isWhiteSpace()) // skip whitespace
            || eventType == XMLStreamConstants.SPACE
            || eventType == XMLStreamConstants.PROCESSING_INSTRUCTION
            || eventType == XMLStreamConstants.COMMENT) {
            eventType = next();
         }
         if (eventType != XMLStreamConstants.START_ELEMENT && eventType != XMLStreamConstants.END_ELEMENT) {
             throw new XMLStreamException("expected start or end tag", getLocation());
         }
         return eventType;    
    }
    
    public int getNamespaceCount() {
        if (state == STATE_FIRST_RUN && rootPrefixes != null) {
            return rootPrefixes.size();
        } else {
            return getParent().getNamespaceCount();
        }
    }
    
    public String getNamespacePrefix(int i) {
        if (state == STATE_FIRST_RUN && rootPrefixes != null) {
            return (String) rootPrefixes.get(i);
        } else {
            return getParent().getNamespacePrefix(i);
        }
    }
    
    public String getNamespaceURI(int i) {
        if (state == STATE_FIRST_RUN && rootPrefixes != null) {
            return getParent().getNamespaceContext().getNamespaceURI(rootPrefixes.get(i));
        } else {
            return getParent().getNamespaceURI(i);
        }
    }
    
    public String getNamespaceURI(String prefix) {
        if (state == STATE_FIRST_RUN) {
            return getParent().getNamespaceContext().getNamespaceURI(prefix);
        } else {
            return getParent().getNamespaceURI(prefix);
        }
    }

    public boolean isStartElement() {
        return event == START_ELEMENT;
    }

    public boolean isEndElement() {
        return event == END_ELEMENT;
    }

    public boolean isCharacters() {
        return event == CHARACTERS;
    }

    public boolean isWhiteSpace() {
        return event == SPACE;
    }

    public boolean hasName() {
        return (event == START_ELEMENT || event == END_ELEMENT);
    }

    public boolean hasText() {
        return (event == CHARACTERS || event == DTD || event == ENTITY_REFERENCE
                        || event == COMMENT || event == SPACE);
    }

}
