/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.workflow.api;



public interface IControlAPI {

	/**
	 * Start a new process instance given the process name ( process definition
	 * name )
	 * 
	 * @param processName
	 *           the process id
	 * @return the newly created instance process instance id
	 */
	public Long startByProcessName(String processName);

	/**
	 * Start a new process instance given the process name ( process definition name )
	 * and a list of process variables. If these variables are not defined, they will be 
	 * automatically created for the process and initialized with the values in input for 
	 * this process instance
	 * 
	 * @param processName   the process id 
	 * @param  vars   the list of variables we want to set on the process instance
	 * @return the newly created instance process instance id
	 */
	public Long startByProcessName(String processName, Variable[] vars);
	
	/**
	 * Abort the process instance given the process instance id (processInstanceId )
	 * 
	 * 
	 * @param processInstanceId
	 *          the process instance Id
	 */
	public void abort(long processInstanceId);
	
	/**
	 * Restart the process instance given the process instance id ( processInstanceId )
	 * and the list of the messages changed by the user
	 * 
	 * @param processInstanceId   The process instance Id
	 * @param modifiedMessages   The messages changed by the user
	 */
	public void restart(long processInstanceId, ModifiedMessage[] modifiedMessages);

	
	/** NOT FINISHED SO NOT PUBLISH IN PUBLIC API
	 * Set a flag for process instance, during process execution if an automatic
	 * task find the process
	 * 
	 * @param processInstanceId   the process instance Id
	 * @deprecated
	 * TO REVIEW AND NOT YET IMPLEMENTED
	
	public void stopOnAutomaticTask(long processInstanceId, boolean stop);
	 */

	
	
	/**
	 * Execute the task called "task" in the instance identified by
	 * "processInstanceId", passing a set of variables. If the task 
	 * instance is not associated to the task instance, an exception
	 * is generated
	 * 
	 * @param processInstanceId
	 *            The process Instance Id
	 * @param taskName
	 *            The task to execute
	 * @param vars
	 *            the array of variables, set to null if you don't want pass
	 *          variables
	 *   
	 */
	public void executeTask(long processInstanceId, String taskName, Variable[] vars);
	
	/**
	 * Start the task called "taskName" in the instance identified by
	 * "processInstanceId", passing a set of variables.If the task 
	 * instance is not associated to the task instance, an exception
	 * is generated. The difference with the previous method is
	 * that this one does not complete the task
	 * 
	 * @param processInstanceId
	 *            The process Instance Id
	 * @param taskName
	 *            The task to execute
	 * @param vars
	 *            the array of variables, set to null if you don't want pass
	 *          variables
	 */
	public void startTask(long processInstanceId, String taskName, Variable[] vars);

	/**
	 * Execute the task called "task" specifying the "task status" in the instance
	 * identified by "processInstanceId", passing a set of variables.If the task 
	 * instance is not associated to the task instance, an exception
	 * is generated.
	 * The name of the "task status variable" is computed according to the parameter
	 * "Task Status Variable Prefix" defined in the section "BPM Gateway Code Generator"
	 * of the Spagic preferences in Spagic Studio.
	 * 
	 * @param processInstanceId
	 *           The process Instance Id
	 * @param taskName
	 *            The task to execute
	 * @param taskStatus
	 *            The status of the task
	 * @param vars
	 *            the array of variables, set to null if you don't want pass
	 *          variables
	 */
	public void executeTaskWithStatus(long processInstanceId, String taskName, String taskStatus, Variable[] vars);

	/**
	 * Execute the task called identified by "taskId" in the instance identified
	 * by "processInstanceId", passing a set of variables.If the task 
	 * instance is not associated to the task instance, an exception
	 * is generated.
	 * 
	 * @param processInstanceId
	 *            The process Instance Id
	 * @param taskId
	 *            The task id
	 * @param vars
	 *            the array of variables, set to null if you don't want pass
	 *          variables
	 */
	public void executeTask(long processInstanceId, long taskId, Variable[] vars);

	/**
	 * Start the task called identified by "taskId" in the instance identified
	 * by "processInstanceId", passing a set of variables.If the process 
	 * instance is not associated to the task instance, an exception
	 * is generated.
	 * 
	 * @param processInstanceId
	 *            The process Instance Id
	 * @param taskId
	 *            The task id
	 * @param vars
	 *            the array of variables, set to null if you don't want pass
	 *          variables
	 */
	public void startTask(long processInstanceId, long taskId, Variable[] vars);

	/**
	 * Execute the task called identified by "taskId" in the instance identified
	 * by "processInstanceId" specifying "TaskStatus", passing a set of variables.
	 * If the process instance is not associated to the task instance, an exception
	 * is generated.
	 * @param processInstanceId
	 *            The process Instance Id
	 * @param taskId
	 *            The task id
	 * @param taskStatus
	 *            The status of the task
	 * @param vars
	 *            the array of variables, set to null if you don't want pass
	 *          variables
	 */
	public void executeTaskWithStatus(long processInstanceId, long taskId, String taskStatus, Variable[] vars);

	/**
	 * Execute the task identified by "taskId" by the given "actor" in the
	 * instance identified by "processInstanceId", passing a set of variables.
	 * If the process instance is not associated to the task instance, an exception
	 * is generated.
	 * 
	 * @param actor
	 *           The actor performing the task
	 * @param processInstanceId
	 *           The process Instance Id
	 * @param taskId
	 *           The task id
	 * @param vars
	 *           the array of variables, set to null if you don't want pass
	 *          variables
	 */
	public void executeTaskByActor(String actor, long processInstanceId, long taskId, Variable[] vars);

	/**
	 * Start the task identified by "taskId" by the given "actor" in the
	 * instance identified by "processInstanceId", passing a set of variables.
	 * If the process instance is not associated to the task instance, an exception
	 * is generated.
	 * 
	 * @param actor
	 *           The actor performing the task
	 * @param processInstanceId
	 *           The process Instance Id
	 * @param taskId
	 *           The task id
	 * @param vars
	 *           the array of variables, set to null if you don't want pass
	 *          variables
	 */
	public void startTaskByActor(String actor, long processInstanceId, long taskId, Variable[] vars);

	/**
	 * Execute the task called "task" by the given "actor" in the instance
	 * identified by "processInstanceId", passing a set of variables.
	 * If the process instance is not associated to the task instance, an exception
	 * is generated.
	 * @param actor
	 *           The actor performing the task
	 * @param processInstanceId
	 *           The process Instance Id
	 * @param taskName
	 *           The task to execute
	 * @param vars
	 *           the array of variables, set to null if you don't want pass
	 *          variables
	 */
	public void executeTaskByActor(String actor, long processInstanceId, String taskName, Variable[] vars);

	/**
	 * Start the task called "task" by the given "actor" in the instance
	 * identified by "processInstanceId", passing a set of variables.
	 * If the process instance is not associated to the task instance, an exception
	 * is generated.
	 * 
	 * @param actor
	 *           The actor performing the task
	 * @param processInstanceId
	 *           The process Instance Id
	 * @param taskName
	 *           The task to execute
	 * @param vars
	 *           the array of variables, set to null if you don't want pass
	 *          variables
	 */
	public void startTaskByActor(String actor, long processInstanceId, String taskName, Variable[] vars);
	

	/**
	 * Update the task instance status saving the completion percentage.
	 * If the process instance is not associated to the task instance, an exception
	 * is generated.
	 * @param processInstanceId Process instance id
	 * @param taskName The task to update
	 * @param completionPercentage New completion percentage
	 */
	public void advanceTaskCompletion(long processInstanceId, String taskName, int completionPercentage);

	/**
	 * Update the task instance status saving the completion percentage.
	 * If the process instance is not associated to the task instance, an exception
	 * is generated.
	 * @param processInstanceId Process instance id
	 * @param taskId The task instance identifier
	 * @param completionPercentage New completion percentage
	 */
	public void advanceTaskCompletion(long processInstanceId, long taskId, int completionPercentage);
	

	/**
	 * Reject a task instance, undoing a previously accepted task
	 * If the process instance is not associated to the task instance, an exception
	 * is generated.
	 * @param processInstanceId Process instance id
	 * @param taskId Task instance id
	 */
	public void rejectTask(long processInstanceId, long taskId);
	
	/**
	 * Reject a task instance, undoing a previously accepted task
	 * If the process instance is not associated to the task instance, an exception
	 * is generated.
	 * @param processInstanceId Process instance id
	 * @param taskId Task instance id
	 * @param actorId the actor name
	 */
	public void rejectTaskByActor(String actorId, long processInstanceId, long taskId);
	

	/**
	 * Set a value of a global variable on process instance.
	 * @param processInstanceId
	 * @param variableName
	 *           the name of variable to set
	 * @param variableValue
	 *           the value of the variable to set
	 */
	public void setGlobalVariable(long processInstanceId, String variableName, Object variableValue);

	/**
	 * Get the value of the specified global variables on process instance.
	 * 
	 * @param processInstanceId
	 * @param variableName
	 *           the name of variable to set
	 */
	public Variable getGlobalVariable(long processInstanceId, String variableName);

	/**
	 * Get the values of all global variables on process instance.
	 * 
	 * @param processInstanceId the id of the Process Instance
	 */
	public Variable[] getGlobalVariables(long processInstanceId);

	/**
	 * Set the values of some global variables on process instance.
	 * 
	 * @param processInstanceId
	 *          Process instance id
	 * @param variables
	 *          Variables values
	 */
	public void setGlobalVariables(long processInstanceId, Variable[] variables);

	/**
	 * Establish dinamically a link between two different process instances.
	 * 
	 * @param firstProcessInstanceId
	 * 			the id of the first process instance to be linked
	 * @param secondProcessInstanceId
	 * 			the id of the second process instance to be linked
	 */
	public void linkProcesses(long firstProcessInstanceId, long secondProcessInstanceId);

	/**
	 * Get all the variables assigned to a specific task on a process instance.
	 * If the process instance is not associated to the task instance, an exception
	 * is generated.
	 * 
	 * @param processInstanceId
	 * 			the id of the process instance
	 * @param taskName
	 *          the name of the task
	 */
	public Variable[] getTaskVariables(long processInstanceId, String taskName);

	/**
	 * Get a variable assigned to a specific task on a process instance.
	 * If the process instance is not associated to the task instance, an exception
	 * is generated.
	 * 
	 * @param processInstanceId
	 * 		the id of the process instance
	 * @param taskName
	 *          the name of the task
	 * @param variableName
	 *           the name of variable to set
	 */
	public Variable getTaskVariable(long processInstanceId, String taskName, String variableName);

	/**
	 * Set a value of a "task" variable on process instance.
	 * If the process instance is not associated to the task instance, an exception
	 * is generated.
	 * 
	 * @param processInstanceId
	 * 			the id of the process instance
	 * @param taskName
	 *          the name of the task
	 * @param variableName
	 *          the name of variable to set
	 * @param variableValue
	 *           the value of the variable to set
	 */
	public void setTaskVariable(long processInstanceId, String taskName, String variableName, Object variableValue);

	/**
	 * Set the values of some task variables on process instance.
	 * If the process instance is not associated to the task instance, an exception
	 * is generated.
	 * 
	 * @param processInstanceId
	 *          Process instance id
	 * @param variables
	 *          Variables values
	 */
	public void setTaskVariables(long processInstanceId, String taskName, Variable[] variables);
	
	/**
	 * Set the percentage completion of the task, if the actor has the grant to do it.
	 * If the process instance is not associated to the task instance, an exception
	 * is generated.
	 * 
	 * @param actorId
	 * 			the name of the actor
	 * @param processInstanceId
	 *          Process instance id
	 * @param taskId
	 *          the id of the task instance
	 * @param  completionPercentage
	 * 				the percentage to set to the task
	 * 			       
	 */
	
	public void advanceTaskCompletionByActor(String actorId,long processInstanceId, long taskId, int completionPercentage);
	
	/**
	 * Set the percentage completion of the task, if the actor has the grant to do it.
	 * If the process instance is not associated to the task instance, an exception
	 * is generated.
	 * 
	 * @param actorId
	 * 			the name of the actor
	 * @param processInstanceId
	 *          Process instance id
	 * @param taskName
	 *          the name of the task 
	 * @param  completionPercentage
	 * 				the percentage to set to the task
	 * 			       
	 */
	
	public void advanceTaskCompletionByActor(String actorId,long processInstanceId, String taskName, int completionPercentage);
}
