/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.workflow.api;

import java.util.Date;

import javax.activation.DataHandler;

/**
 * @author daniele
 * 
 */
public interface IQueryAPI {

	//
	// --------> STATIC QUERIES
	// 
	/**
	 * Return the SVG graph of the specified process id. The javascript callback
	 * function specified by "onClickCallbackFuntion" is used for interaction with
	 * each step.
	 * 
	 * @param processId
	 *          Process Id
	 * @return The SVG graph.
	 */
	public String getProcessGraph(long processId);

	/**
	 * Get the process definition identified by "processId"
	 * 
	 * @param processId   The process ID 
	 * @return The process definition
	 */
	public Process getProcess(long processId);


	/**
	 * Gets the latest  process definition for process identified by processName. 
	 * 
	 * @param processName   The process Name
	 * @return The latest definition for process identified by "processName"
	 */
	public Process getProcess(String processName);

	/**
	 *  Get all processes and subprocess
	 *
	 *
	 * @return all the process definitions
	 */	
	public Process[] getProcesses();

	/**
	 *  Get all processes,choosing if  
	 *  getting also the processes that are subprocess, or not
	 *
	 * @param onlyStandalone : if true, you won't get processes that are subprocess
	 * @return all the process definitions
	 */	
	public Process[] getProcesses(boolean onlyStandalone);

	/**
	 *  Get all processes containing a predefined static attribute
	 *
	 * @param attributeName the name of the static attribute used as filter 
	 * @param attributeValue the value of the static attribute
	 * @param onlyStandalone : if true, you won't get processes that are subprocess
	 * @return all the process definitions
	 */	
	 
	public Process[] getProcessesByFilter(String attributeName, String attributeValue, boolean onlyStandalone);

	
	/**
	 *  Gets the latest  processes definition
	 *
	 * 
	 * @return all the latest processes definitions
	 */	
	public Process[] getLatestDefinitionProcesses();

	/**
	 * Gets the latest definition processes, containing a predefined static attribute.If onlyStandalone=true, it retrieves
	 * only the processes that can be started as standalone processes (for example
	 * it doesn't retrieve subprocesses).
	 * 
	 * @param onlyStandalone
	 *          If true returns only standalone processes
	 * @return The latest definition processes
	 */
	public Process[] getLatestDefinitionProcesses(boolean onlyStandalone);

	/**
	 * Gets the latest definition processes. If onlyStandalone=true, it retrieves
	 * only the processes that can be started as standalone processes (for example
	 * it doesn't retrieve subprocesses).
	 * 
	 * @param attributeName the name of the static attribute used as filter 
	 * @param attributeValue the value of the static attribute
	 * @param onlyStandalone
	 *          If true returns only standalone processes
	 * @return The latest definition processes
	 */
	
	public Process[] getLatestDefinitionProcessesByFilter(String attributeName, String attributeValue,
			boolean onlyStandalone);

	/**
	 * Retrieves all static process properties defined during process design
	 * @param processId Process identifier
	 * @return List of all static process properties. Returns an empty list if no property
	 * is defined
	 */
	public Variable[] getProcessProperties(long processId);

	/**
	 * Retrieves all static process properties defined during process design
	 * @param processName Process name
	 * @return List of all static process properties. Returns an empty list if no property
	 * is defined
	 */
	public Variable[] getProcessProperties(String processName);

	/**
	 * Gets all the human task for the given process
	 * 
	 * @param processId   The process identifier
	 * @return   The array of task definition
	 */
	public Task[] getHumanTask(long processId);

	/**
	 * Gets all the human task for the given process
	 * 
	 * @param processName   The process name
	 * @return   The array of task definition
	 */
	public Task[] getHumanTask(String processName);

	/**
	 * Gets all the human task for the given process
	 * 
	 * @param process   The process object
	 * @return   The array of task definition
	 */
	public Task[] getHumanTask(Process process);

	/**
	 * Gets the human task for the given process, called taskName
	 * 
	 * @param processId   The process identifier
	 * @param taskName   The task name
	 * @return   The task definition
	 */
	public Task getHumanTask(long processId, String taskName);

	/**
	 * Gets all the task of the process that are subprocess
	 * 
	 * @param processId   The process identifier
	 * @return   The array of Subprocess Task
	 */
	public SubProcessTask[] getSubprocessTasks(long processId);

	/**
	 * Gets all the task of the process that are subprocess of the given process
	 * 
	 * @param processName   The process name
	 * @return   The array of Subprocess Task
	 */
	public SubProcessTask[] getSubprocessTasks(String processName);

	/**
	 * Gets all the task of the process that are subprocess of the given process
	 * 
	 * @param process   The process 
	 * @return   The array of Subprocess Task
	 */
	public SubProcessTask[] getSubprocessTasks(Process process);

	/**
	 * Retrieves all processes that are used as subprocesses within the process
	 * identified by processId.
	 * 
	 * @param processId
	 *          Process identifiers
	 * @return The process definitions of the processes used as subprocesses
	 *         within a process.
	 */
	public Process[] getSubProcesses(long processId);

	/**
	 * Retrieves the subprocesses used in a main process, given its name.
	 * 
	 * @param processName
	 *          Process name
	 * @return The process definitions of the processes used as subprocesses
	 *         within a process.
	 */
	public Process[] getSubProcesses(String processName);

	/**
	 * Get the values of the condition associated to outgoing transitions of a task
	 * @param processName   The process Name
	 * @param taskName   The task Name
	 * @return  the values of the conditions associted to the process 
	 *
	 */
	public String[] getHumanTaskOutgoingValues(String processName, String taskName);

	/**
	 * Get the values of the condition associated to outgoing transitions of a task
	 * @param processId   The process Id
	 * @param taskName   The task Name
	 * @return  the values of the conditions associted to the process 
	 * 
	 */
	public String[] getHumanTaskOutgoingValues(long processId, String taskName);

	//
	// ---------> DYNAMIC QUERIES ( ProcessInstances and TaskInstances )
	//

	/**
	 * Retrieves all process instances.
	 * 
	 * @return A list of process instances.
	 */
	public ProcessInstance[] getProcessInstances();

	/**
	 * Retrieves all process instances of a specified process.
	 * 
	 * @param processId
	 *          Process Id
	 * @return   The array of Process Instance
	 */
	public ProcessInstance[] getProcessInstances(long processId);

	/**
	 * Retrieves all process instances of a specified process.
	 * 
	 * @param processName
	 *          Process Name
	 * @return   The array of Process Instance
	 */
	public ProcessInstance[] getProcessInstances(String processName);

	/**
	 * Retrieves all process instances that are subprocess instance of the given process instance
	 * 
	 * @param parentProcessInstanceId - The parent process instance id
	 *          
	 * @return A list of process instances.
	 */
	public ProcessInstance[] getSubProcessInstances(long parentProcessInstanceId);

	/**
	 * Get the process instance identified by processInstanceId
	 * 
	 * @param processInstanceId   The parent process instance id
	 * @return The Process Instance
	 */
	public ProcessInstance getProcessInstanceById(long processInstanceId);

	/**
	 * <code>getHistoricListofTasks</code> Returns all the
	 * human task instances that are completed and generated 
	 * for a given process instance. TaskInstances are chronologically 
	 * ordered. The first elements of the list are the task instances 
	 * that have to be completed and the last are the oldest 
	 * taskinstances completed.
	 * 
	 *
	 * @param processInstanceId a <code>long</code> the id of the
	 * process instance
	 * @return a <code>TaskInstance[]</code> the descriptors of the
	 * tasksInstances of the given process, chronologically ordered.
	 */
	public TaskInstance[] getHistoricListofTasks(final long processInstanceId);
	
	
	
	
	
	
	/**
	 * Retrieves all the generated human task instances  for a specified process instance.
	 * 
	 * @param processInstanceId
	 *          Process instance id
	 * @return Array of Task instances
	 */
	public TaskInstance[] getHumanTaskInstances(long processInstanceId);

	/**
	 * Retrieves all generated human task instances for a specified actor id, filtering
	 * on a set of variables. Only the process instances that contains the specified
	 * set of variables, with the specified values, are returned.
	 * 
	 * @param actorID Identifier of the user that must execute the tasks
	 * @param variables Set of variables for filtering
	 * @return Array of Task instances
	 */
	public TaskInstance[] getHumanTaskInstancesForActor(String actorID, Variable[] variables);

	/**
	 * Retrieves all generated human task instances 
	 *  for a specified process instance.
	 * 
	 * @param processInstanceId
	 *          Process instance id
	 * @param actorID
	 *          Identifier of the user that must execute the tasks
	 * @return Array of Task instances
	 */
	public TaskInstance[] getHumanTaskInstancesForActorAndProcess(long processInstanceId, String actorID);

	/**
	 * Retrieves all the generated human task instances  
	 *  for a specified actor.
	 * 
	 * 
	 * @param actorID the user name 
	 *          
	 * @return Array of Task instances
	 */
	public TaskInstance[] getHumanTaskInstancesForActor(String actorID);

	/**
	 * Retrieves all generated human task instances 
	 *  for a specified process instance filtering on the task attribute
	 * 
	 * @param processInstanceId
	 *          Process instance id
	 * @param attributeName  The name of the attribute to use as filter
	 * @param attributeValue  The value to use in filter
	 * @return Array of Task instances
	 */
	public TaskInstance[] getHumanTaskInstances(long processInstanceId, String attributeName, String attributeValue);

	/**
	 * Retrieves the SVG Graph for the given processInstanceId
	 * @param processInstanceId
	 * 			the istance of Process
	 * @return the SVG Graph for the given processInstanceId
	 */
	public String getProcessInstanceGraph(long processInstanceId);

	/**
	 * Retrieves all identifiers of process instances linked to the provided one.
	 * 
	 * @param processInstanceId 
	 *          Process instance for starting the search
	 * @return Identifiers of linked process instances.
	 */
	public ProcessInstance[] getLinkedProcessInstances(long processInstanceId);

	/**
	 * Retrieve the value of the variable related to the "business status" of the
	 * task instance
	 * 
	 * @param processInstanceId
	 * 			the id of Process Instance
	 * @param taskInstanceId
	 * 			the id of Task Instance
	 * @return The value of the variable related to the "business status" of the
	 *         task instance
	 */
	public String getTaskInstanceStatus(long processInstanceId, long taskInstanceId);

	/**
	 * Retrieve the value of the variable related to the "business status" of a
	 * particular process instance
	 * 
	 * @param processInstanceId the id of Process instance
	 * @return The value of the variable related to the "business status" of the
	 *         task instance
	 */
	public String getProcessInstanceStatus(long processInstanceId);

	
	/**
	 * Get name of the generated tasks in the given processInstanceId
	 * @param processInstanceId  The process instance Id
	 * @return the array of name of the generated tasks 
	 */
	public String[] getNextTasksNames(long processInstanceId);

	/**
	 * Get the binary file associated to task if there's one
	 * @param processName 
	 * 		 	The process name
	 * @param taskName 
	 * 			 The task name
	 * @return the binary file associated to task
	 */
	public DataHandler getTaskDocumentation(String processName, String taskName);

	//
	// --------> TASK LIST
	// 
	/**
	 * Retrieve all the task instances that are in the states of generated or
	 * started.
	 * 
	 * @return array of task instances  
	 */
	public TaskInstance[] getGeneratedHumanTaskInstances();

	/**
	 * Retrieve all the task instances filtered by creation date, due date,
	 * process name, task name, state or process instance id. The instances
	 * returned are in the states of generated or started.
	 * 
	 * @param processName
	 * 			the name of the Process
	 * @param taskName
	 * 		the name of the Task
	 * @param createFrom
	 * 		the date of creation to search from
	 * @param createTo
	 * 	    the date of creation to search to
	 * @param dueDateFrom
	 * 		the date of expiration to search from
	 * @param dueDateTo
	 * 		the date of expiration to search to
	 * @param state
	 * 		the state of the process
	 * @param processInstanceId
	 * 		the id of the process instance
	 * @return TaskInstance[] the array of task instances found
	 */
	public TaskInstance[] getGeneratedHumanTaskInstances(String processName, String taskName, Date createFrom,
			Date createTo, Date dueDateFrom, Date dueDateTo, int state, long processInstanceId);

	/**
	 * Retrieve all the task instances that are assigned to the user
	 * 
	 * @param actorId 
	 * 			the user name
	 * @return TaskInstance[] the array of task instances found
	 */
	public TaskInstance[] getHumanTaskInstancesAssignedToActor(String actorId);

	/**
	 * <code>getGeneratedHumanTaskInstances</code> Returns all the
	 * generated human task instances for a given process instance,
	 * whatever the owner of the task is.
	 *
	 * @param processInstanceId a <code>long</code> the id of the
	 * process instance
	 * @return a <code>TaskInstance[]</code> the descriptors of the
	 * tasks of the given process.
	 */
	public TaskInstance[] getGeneratedHumanTaskInstances(long processInstanceId);

}
