/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.components.bpm.deploy;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import org.dom4j.Document;
import org.eclipse.emf.common.util.EList;
import org.eclipse.stp.im.BasicProperty;
import org.eclipse.stp.im.ConfigurableElement;
import org.eclipse.stp.im.Process;
import org.eclipse.stp.im.Property;
import org.eclipse.stp.im.Service;
import org.eclipse.stp.im.ServiceBinding;
import org.eclipse.stp.im.Step;
import org.eclipse.stp.im.StpIntermediateModel;
import org.eclipse.stp.im.Transition;
import org.eclipse.stp.im.Variable;
import org.eclipse.stp.im.constants.ImConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.spagic3.components.bpm.deploy.utils.DeploymentUtils;

public class JBPMPreprocessor extends AbstractImProcessor {

	//
	private static final String JBPM_TECHNOLOGY = "JBPM";
	private static final String SERVICE_StartService = "StartService";
	private static final String SERVICE_EndService = "EndService";
	private static final String SERVICE_TaskService = "TaskService";
	private static final String SERVICE_RouterService = "RouterService";
	private static final String SERVICE_ParallelService = "ParallelService";
	private static final String SERVICE_JoinService = "JoinService";
	private static final String SERVICE_SubprocessService = "SubprocessService";
	private static final String SB_Start = "Start";
	private static final String SB_End = "End";
	private static final String SB_HumanTask = "HumanTask";
	private static final String SB_Fork = "Fork";
	private static final String SB_Join = "Join";
	private static final String SB_Subprocess = "Subprocess";
	private static final String SB_SetStatusTask = "SetStatusTask";
	private static final String SB_Decision = "Decision";
	private static final String SB_AutomaticTask = "AutomaticTask";
	private static final String SB_NotifyTask = "NotifyTask";
	private static final String DefaultNotifierClassName = "org.spagic.workflow.handler.NotifyHandler";
	private static final String NOTIFIER_TASK_FOR = "NOTIFIER_TASK_FOR_";
	private static final String IS_STEP_GENERATED_BY_PREPROCESSOR = "IS_STEP_GENERATED_BY_PREPROCESSOR";
	private static final String OUTGOING_CONDITION_VALUE_PREFIX = "OutGoingValue";
	private static final String JBPM_STEP_NAME_PROPERTY = "jbpm_step_name";
	private static final String FORM_SCHEMA = "FormSchema";
	private static final String FORM_ID = "FormId";
	private static final String FORM_TYPE = "FormType";
	private static final String ORBEON_INTEGRATON_PARAMETER = "OrbeonBaseUrl";
	
	
	
	private String name;
	private String technology;
	private String runtimeId = null;
	private Properties prefs = null;
	
	private static Logger logger = LoggerFactory.getLogger(JBPMPreprocessor.class);
	
	public JBPMPreprocessor(Properties prefs) {
		this.name = "JBPMPreprocessor";
		this.runtimeId = "runtime.jbpm";
		this.technology = JBPM_TECHNOLOGY;
		this.prefs = prefs;
	}

	public String getName() {
		return name;
	}

	

	public String getTechnology() {
		return technology;
	}

	@Override
	public boolean handleAutomaticVersioning() {
		return true;
	}

	

	

	public String normalizeName(String s) {
		s = s.replace(' ', '_');
		s = s.replace('?', '_');
		s = s.replace('\'', '_');
		s = s.replace('&', '_');
		return s;
	}

	public String getVariableName(String prefix, String name) {
		return prefix + normalizeName(name);
	}

	

	public void process(StpIntermediateModel im) {
		for (Process process : im.getProcessCollection().getProcesses()) {
			handleImProcess(im, process);
		}
	}

	public void handleImProcess(StpIntermediateModel im, Process process) {

		// Create the technology property
		createTechnologyProperty(process, JBPM_TECHNOLOGY);

		// Create the Orbeon Integration Property
		createOrbeonIntegrationProperty(process);

		createAutomaticVariables(im, process);

		// Put the expressions for the router nodes on the outgoing router transitions
		createExpressionOnTransitions(im, process);

		// Automatically detect some nodes types such as: start, end, fork, router, etc
		completeAnnnotations(im, process);

		detectDecisionAndForkNode(im, process);

		completeSubProcessInformation(im, process);

		completeTimerInformation(im, process);

		checkNotSignallingTask(im, process);

		completeJbpmNamePropertyOnStep(im, process);

		completeHasConditionalExitFlag(im, process);

		completeSetStatusTaskValue(im, process);

		completeSchemaInformation(im, process);

	}

	public void completeSetStatusTaskValue(StpIntermediateModel im, Process aProcess) {
		for (Step aStep : aProcess.getSteps()) {
			ServiceBinding sb = getRightStepBinding(aStep);

			if (sb.getName().equalsIgnoreCase(SB_SetStatusTask)) {
				BasicProperty valueProperty = (BasicProperty) aStep.getProperties().get("Value");

				String valueValue = valueProperty.getValue();
				if (valueValue.equalsIgnoreCase("${FROM_INCOMING_TRANSITION_LABEL}")) {
					EList<Transition> incomingTrasition = aStep.getTargetTransitions();

					if (incomingTrasition.size() > 0) {
						String transitionLabel = getTransitionProperty(incomingTrasition.get(0), "TransitionLabel");
						valueProperty.setValue(transitionLabel);
					}
				}
			}
		}
	}
	
	public void completeSchemaInformation(StpIntermediateModel im, Process aProcess) {
		for (Step aStep : aProcess.getSteps()) {
			ServiceBinding sb = getRightStepBinding(aStep);

			if (sb.getName().equalsIgnoreCase(SB_HumanTask)) {
				BasicProperty formTypeProperty = (BasicProperty) aStep.getProperties().get(FORM_TYPE);
				BasicProperty formProperty = (BasicProperty) aStep.getProperties().get(FORM_SCHEMA);

				String formTypeName = (formTypeProperty == null) 
					? "NoForm"
							: formTypeProperty.getValue();
				
				
				if ( ("NoForm".equalsIgnoreCase(formTypeName))) {
					// First case: there is NO form at all
					continue;
				}				
				else if ("OrbeonForm".equals(formTypeName) || "PdfForm".equals(formTypeName)) {
					// those are the forms used with Orbeon.
					setPropertyOnStep(aStep, FORM_ID, "___NoIdRequired__");
				}
				else if ((formProperty != null) &&
					(formProperty.getValue() != null) && (formProperty.getValue().length() != 0)) {
					// Here we have a dynamic form!
					assert "Dynamic".equals(formTypeName);
					
					String schemaValue = formProperty.getValue();
					
					String realSchemaValue = DeploymentUtils.getSchemaFileLocation(schemaValue);
					// Retrieve the schema identifier reading the Schema definition file
					File schemaFile = new File(realSchemaValue);
					
					// The schema file may not exist
					if (schemaFile.exists()) {
						Document document = DeploymentUtils.fetchDocument(schemaFile);
						String idMetadata = DeploymentUtils.evalXPathAsString(document, "metadatadefinition/@id");

						setPropertyOnStep(aStep, FORM_ID, idMetadata);
					} else {
						throw new RuntimeException("Step [" + aStep.getName()
								+ "] - Schema file [" + realSchemaValue + "] not found on workspace");
					}					
					
				}
			}
		}
	}
	
	

	
	

	public void completeHasConditionalExitFlag(StpIntermediateModel im, Process aProcess) {

		//
		// I must put the flag on the node that has a transition for which
		// the target of the transition is a router service
		//
		for (Step aStep : aProcess.getSteps()) {
			EList<Transition> outcomingTransitions = aStep.getSourceTransitions();

			for (Transition aTransition : outcomingTransitions) {
				ServiceBinding sb = getRightStepBinding(aTransition.getTarget());
				if (sb.getName().equalsIgnoreCase(SB_Decision)) {
					setPropertyOnStep(aStep, ImConstants.HAS_STEP_CONDITIONAL_EXIT_FLAG, "true");
				}

			}
		}
	}

	public void completeJbpmNamePropertyOnStep(StpIntermediateModel im, Process aProcess) {
		for (Step aStep : aProcess.getSteps()) {
			setPropertyOnStep(aStep, JBPM_STEP_NAME_PROPERTY, aStep.getName());
		}
	}

	public void checkNotSignallingTask(StpIntermediateModel im, Process aProcess) {

		for (Step aStep : aProcess.getSteps()) {

			ServiceBinding sb = getRightStepBinding(aStep);
			if (sb == null){
				 sb = getRightStepBinding(aStep);
			}
			EList<Transition> outcomingTransition = aStep.getSourceTransitions();
			if (sb.getName().equalsIgnoreCase(SB_HumanTask) && (outcomingTransition.size() == 0)) {
				setPropertyOnStep(aStep, "NotSignalTask", "true");
			}
		}
	}

	public void completeSubProcessInformation(StpIntermediateModel im, Process aProcess) {
		for (Step aStep : aProcess.getSteps()) {
			Service serviceModel = aStep.getServiceModel();
			if (serviceModel != null && serviceModel.getServiceName().equalsIgnoreCase(SERVICE_SubprocessService)) {

				setPropertyOnStep(aStep, ImConstants.IS_STEP_SUBPROCESS, "true");
			}
		}
	}

	public void completeTimerInformation(StpIntermediateModel im, Process aProcess) {
		List<Step> notifierSteps = new ArrayList<Step>();
		for (Step aStep : aProcess.getSteps()) {
			if (hasTimer(aStep)) {

				String timerName = uniqueIdentifierForTimer(aStep);
				setPropertyOnStep(aStep, "TimerName", timerName);

				if (isAbsoluteTimer(aStep)) {
					Variable var = factory.createVariable();

					String varName = "AbsoluteDateFor" + timerName;
					var.setName(varName + "->" + "java.util.Date(" + getAbsoluteDateTimerAsString(aStep) + ")");
					aProcess.getProcessVariables().add(var);

					setPropertyOnStep(aStep, "NameOfAbsoluteDateVariable", varName);

				}
				EList<Transition> outcomingTransitions = aStep.getSourceTransitions();

				// In the default case the first outcoming transition is the transition
				// for timer
				String onExpirationAction = getStepProperty(aStep, "ExpirationAction");
				String nameOfDestination = null;

				if (onExpirationAction == null || onExpirationAction.equalsIgnoreCase("DefaultTransition")) {
					if (outcomingTransitions.size() > 0) {
						nameOfDestination = outcomingTransitions.get(0).getSource().getName() + "to"
								+ outcomingTransitions.get(0).getTarget().getName();
					}
					boolean founded = false;
					for (Transition outcomingTransition : outcomingTransitions) {
						if ((getTransitionProperty(outcomingTransition, "IsTransitionForTimerExpriration") != null) && (!founded)) {
							nameOfDestination = outcomingTransition.getSource().getName() + "to"
									+ outcomingTransition.getTarget().getName();
							founded = true;
						}
					}
				} else {
					Step notifierStep = createStep(im, NOTIFIER_TASK_FOR + aStep.getName(), SB_NotifyTask);
					setPropertyOnStep(notifierStep, "NotifierClassName", DefaultNotifierClassName);
					if (outcomingTransitions.size() > 0) {
						createTransitionAndAddToProcess(aProcess, notifierStep, outcomingTransitions.get(0).getTarget());
					}
					createTransitionAndAddToProcess(aProcess, aStep, notifierStep);

					nameOfDestination = aStep.getName() + "to" + notifierStep.getName();
					notifierSteps.add(notifierStep);
				}

				if (nameOfDestination != null) {
					setPropertyOnStep(aStep, "TimerDestination", nameOfDestination);
				}
			}
		}
		
		if (notifierSteps.size() > 0) {
			aProcess.getSteps().addAll(notifierSteps);
		}
	}

	public static boolean hasTimer(Step aStep) {
		BasicProperty timerType = (BasicProperty) aStep.getProperties().get("TimerType");
		boolean retValue = false;
		if (timerType != null) {
			if (timerType.getValue().equalsIgnoreCase("RelativeExpirationTimer")
					|| timerType.getValue().equalsIgnoreCase("AbsoluteExpirationTimer")) {
				retValue = true;
			}
		}
		return retValue;
	}

	public static boolean isAbsoluteTimer(Step aStep) {
		BasicProperty timerType = (BasicProperty) aStep.getProperties().get("TimerType");

		if (timerType != null) {
			return timerType.getValue().equalsIgnoreCase("AbsoluteExpirationTimer");
		}
		return false;
	}

	public static String getAbsoluteDateTimerAsString(Step aStep) {
		BasicProperty absoluteDate = (BasicProperty) aStep.getProperties().get("AbsoluteExpirationDate");
		return absoluteDate.getValue();

	}

	public static String uniqueIdentifierForTimer(Step aStep) {
		return "timer" + aStep.getName();
	}

	public boolean isHumanTask(Step aStep) {
		boolean result = true;
		ServiceBinding sb = getRightStepBinding(aStep);
		if (sb != null) {
			result = sb.getName().equalsIgnoreCase(SB_HumanTask);
		} else {
			boolean isStart = getStepProperty(aStep, ImConstants.IS_STEP_START_EVENT) != null;
			boolean isEnd = getStepProperty(aStep, ImConstants.IS_STEP_END_EVENT) != null;
			boolean isExlusiveGateway = getStepProperty(aStep, ImConstants.IS_EXCLUSIVE_GATEWAY) != null;
			boolean isParallelGateway = getStepProperty(aStep, ImConstants.IS_PARALLEL_GATEWAY) != null;

			result = !(isStart || isEnd || isExlusiveGateway || isParallelGateway);

		}
		return result;
	}

	/**
	 * According to the Spagic preferences, creates automatically some variables to
	 * store the process result and/or the tasks results
	 * @param im The Intermediate model
	 * @param aProcess The process to change
	 */
	public void createAutomaticVariables(StpIntermediateModel im, Process aProcess) {

		String createProcessVariable = prefs.getProperty(DeployPreferenceConstants.P_WF_CREATE_VARIABLE_FOR_PROCESS_STATUS);
		String createTaskStatusVariable = prefs.getProperty(DeployPreferenceConstants.P_WF_CREATE_VARIABLE_FOR_TASK_STATUS);
		String createSubprocessStatusVariable = prefs.getProperty(DeployPreferenceConstants.P_WF_CREATE_VARIABLE_FOR_SUBPROCESS_STATUS);
				

		if (createProcessVariable.equalsIgnoreCase("true")) {
			Variable var = factory.createVariable();
			var.setName(prefs.getProperty(DeployPreferenceConstants.P_WF_PROCESS_STATUS_VARIABLE_NAME));
			aProcess.getProcessVariables().add(var);

			BasicProperty bp = factory.createBasicProperty();
			bp.setKey("ProcessInstanceStatusVariableName");
			bp.setValue(prefs.getProperty(DeployPreferenceConstants.P_WF_PROCESS_STATUS_VARIABLE_NAME));
			aProcess.getProperties().put("ProcessInstanceStatusVariableName", bp);
		}

		if (createTaskStatusVariable.equalsIgnoreCase("true")) {
			String taskStatusVariablePrefix = prefs.getProperty(DeployPreferenceConstants.P_WF_VARIABLE_FOR_TASK_STATUS_PREFIX);
			for (Step aStep : aProcess.getSteps()) {

				if (isHumanTask(aStep)) {

					Variable var = factory.createVariable();
					var.setName(getVariableName(taskStatusVariablePrefix, aStep.getName()));
					aProcess.getProcessVariables().add(var);

					BasicProperty bp = factory.createBasicProperty();
					bp.setKey("TaskInstanceStatusVariableName");
					bp.setValue(getVariableName(taskStatusVariablePrefix, aStep.getName()));
					aStep.getProperties().put("TaskInstanceStatusVariableName", bp);
				}
			}
		}

		if (createSubprocessStatusVariable.equalsIgnoreCase("true")) {
			String subProcessVariablePrefix = prefs.getProperty(DeployPreferenceConstants.P_WF_VARIABLE_FOR_SUBPROCESS_PREFIX);

			for (Step aStep : aProcess.getSteps()) {
				Service serviceModel = aStep.getServiceModel();
				if (serviceModel != null && serviceModel.getServiceName().equalsIgnoreCase(SERVICE_SubprocessService)) {

					String subProcessName = getStepProperty(aStep, "SubProcess");
					Variable var = factory.createVariable();
					var.setName(getVariableName(subProcessVariablePrefix, subProcessName));
					aProcess.getProcessVariables().add(var);

					setPropertyOnStep(aStep, "MappedSubProcessVariableName", 
							 prefs.getProperty(DeployPreferenceConstants.P_WF_PROCESS_STATUS_VARIABLE_NAME));
					setPropertyOnStep(aStep, "MappedInMainProcessVariableName", getVariableName(subProcessVariablePrefix, aStep
							.getName()));

					BasicProperty bp = factory.createBasicProperty();
					bp.setKey("SubprocessStatusVariableName");
					bp.setValue(getVariableName(subProcessVariablePrefix, aStep.getName()));
					aProcess.getProperties().put("SubprocessStatusVariableName", bp);
				}
			}
		}

	}

	public void detectDecisionAndForkNode(StpIntermediateModel im, Process aProcess) {

		String suffixForDecisionNode = prefs.getProperty(DeployPreferenceConstants.P_WF_NAME_SUFFIX_FOR_DECISION_NODE);
		String suffixForForkNode = prefs.getProperty(DeployPreferenceConstants.P_WF_NAME_SUFFIX_FOR_FORK_NODE);
		List<Step> stepToAddToProcess = new ArrayList<Step>();
		for (Step aStep : aProcess.getSteps()) {
			EList<Transition> outcomingTransition = aStep.getSourceTransitions();

			boolean isExclusiveGateway = getStepProperty(aStep, ImConstants.IS_EXCLUSIVE_GATEWAY) != null;
			boolean isParallelGateway = getStepProperty(aStep, ImConstants.IS_PARALLEL_GATEWAY) != null;

			int conditionedTransitions = 0;
			int notConditionedTransitions = 0;
			for (Transition t : outcomingTransition) {
				if (getTransitionProperty(t, "Expression") != null) {
					conditionedTransitions++;
				} else {
					notConditionedTransitions++;
				}
			}

			if (isParallelGateway) {
				continue;
			} else if (isExclusiveGateway) {
				if (conditionedTransitions > 0) {
					int k = 0;
					for (Transition outgoingFromDecisionTransition : aStep.getSourceTransitions()) {
						String transitionLabel = getTransitionProperty(outgoingFromDecisionTransition, "TransitionLabel");

						if (transitionLabel != null) {
							setPropertyOnStep(aStep, OUTGOING_CONDITION_VALUE_PREFIX + "[" + k + "]", transitionLabel);
							k++;
						} // if (transitionLabel != null){
					}// for (Transition outgoingFromDecisionTransition :
					// newStep.getSourceTransitions() ){
				}// if (isTherAtLeastOneConditionedTransition ){
				continue;

			} else {

				Step newStep = null;
				if (outcomingTransition.size() > 1 && (conditionedTransitions > 0)) {

					// There's at least 1 conditioned transition It's a Decision Node

					if (notConditionedTransitions > 1) {
						// It's an error
						throw new RuntimeException("Step [" + aStep.getName()
								+ "] -  There could be at least one Default Transition on a Decision Node ");
					}

					newStep = createStep(im, aStep.getName() + suffixForDecisionNode, SB_Decision);
					newStep.setServiceModel(findServiceName(im, SERVICE_RouterService));
				} else if (outcomingTransition.size() > 1 && (conditionedTransitions == 0)) {
					//
					// If a Step has more than one outcoming transition and at least one
					// is conditiones it's a Fork Node
					//
					newStep = createStep(im, aStep.getName() + suffixForForkNode, SB_Fork);
					newStep.setServiceModel(findServiceName(im, SERVICE_ParallelService));
				}

				if (newStep != null) {

					setPropertyOnStep(newStep, IS_STEP_GENERATED_BY_PREPROCESSOR, "true");
					stepToAddToProcess.add(newStep);
					List<Transition> toRemove = new ArrayList<Transition>();

					for (int i = 0; i < outcomingTransition.size(); i++) {
						Transition aTransition = outcomingTransition.get(i);
						toRemove.add(aTransition);

						createTransitionAndAddToProcess(aProcess, newStep, aTransition.getTarget(), aTransition.getProperties());
					}

					for (Transition t : toRemove) {

						removeTransitionFromProcess(aProcess, t);
						removeFromStepOutcomingTransition(aStep, t);
						removeFromStepIncomingTransition(t.getTarget(), t);
					}

					createTransitionAndAddToProcess(aProcess, aStep, newStep);

					if (conditionedTransitions > 0) {
						int k = 0;
						for (Transition outgoingFromDecisionTransition : newStep.getSourceTransitions()) {
							String transitionLabel = getTransitionProperty(outgoingFromDecisionTransition, "TransitionLabel");

							if (transitionLabel != null) {
								setPropertyOnStep(newStep, OUTGOING_CONDITION_VALUE_PREFIX + "[" + k + "]", transitionLabel);
								k++;
							} // if (transitionLabel != null){
						}// for (Transition outgoingFromDecisionTransition :
						// newStep.getSourceTransitions() ){
					}// if (isTherAtLeastOneConditionedTransition ){
				}//
			}

		}
		aProcess.getSteps().addAll(stepToAddToProcess);

	}

	public void completeAnnnotations(StpIntermediateModel im, Process aProcess) {
		for (Step aStep : aProcess.getSteps()) {

			EList<Transition> outcomingTransition = aStep.getSourceTransitions();
			EList<Transition> incomingTransition = aStep.getTargetTransitions();

			if (getRightStepBinding(aStep) == null) {
				String serviceName = SERVICE_TaskService;
				String serviceBindingName = SB_HumanTask;

				if (getStepProperty(aStep, ImConstants.IS_STEP_START_EVENT) != null) {
					serviceName = SERVICE_StartService;
					serviceBindingName = SB_Start;
				} else if (getStepProperty(aStep, ImConstants.IS_STEP_END_EVENT) != null) {
					serviceName = SERVICE_EndService;
					serviceBindingName = SB_End;
				} else if (getStepProperty(aStep, ImConstants.IS_EXCLUSIVE_GATEWAY) != null) {
					int conditionedTransitions = 0;
					int notConditionedTransitions = 0;
					int k = 0;
					for (Transition t : outcomingTransition) {
						String transitionLabel = getTransitionProperty(t, "TransitionLabel");
						if (getTransitionProperty(t, "Expression") != null) {
							conditionedTransitions++;
							setPropertyOnStep(aStep, OUTGOING_CONDITION_VALUE_PREFIX + "[" + k + "]", transitionLabel);
							k++;
						} else {
							notConditionedTransitions++;
						}
					}

					if (outcomingTransition.size() > 1 && (conditionedTransitions > 0) && (notConditionedTransitions > 1)) {
						// It's an error
						throw new RuntimeException("Step [" + aStep.getName()
								+ "] -  There could be at least one Default Transition on a Decision Node ");
					}
					// if i've more than one outcoming transition it's a decision node
					serviceName = SERVICE_RouterService;
					serviceBindingName = SB_Decision;
				} else if (getStepProperty(aStep, ImConstants.IS_PARALLEL_GATEWAY) != null) {
					if (outcomingTransition.size() > 1 && incomingTransition.size() == 1) {
						// if i've more than one outcoming transition it's a decision node
						serviceName = SERVICE_ParallelService;
						serviceBindingName = SB_Fork;

					} else if (outcomingTransition.size() == 1 && incomingTransition.size() > 1) {
						serviceName = SERVICE_JoinService;
						serviceBindingName = SB_Join;
					}
				} else {
					serviceName = SERVICE_TaskService;
					serviceBindingName = SB_HumanTask;
				}
				ServiceBinding sb = findServiceBindingWithName(im, serviceBindingName);
				Service service = findServiceName(im, serviceName);
				aStep.getStepbindings().add(sb);
				aStep.setServiceModel(service);

			}
		}
	}

	public void createExpressionOnTransitions(StpIntermediateModel im, Process aProcess) {
		String variableBindOnCondition = prefs.getProperty(DeployPreferenceConstants.P_WF_BIND_VARIABLE_FOR_CONDITION);

		if (variableBindOnCondition.equalsIgnoreCase("$ProcessInstanceVariableStatus")) {
			variableBindOnCondition = prefs.getProperty(DeployPreferenceConstants.P_WF_PROCESS_STATUS_VARIABLE_NAME);
		}
		for (Transition aTransition : aProcess.getTransitions()) {
			String transitionLabel = getTransitionProperty(aTransition, "TransitionLabel");
			String exprLanguage = getTransitionProperty(aTransition, "ExpressionLanguage");
			
			if ((exprLanguage != null) && (exprLanguage.equalsIgnoreCase("XPATH"))){
				String xPathVariable = getTransitionProperty(aTransition, "EvaluateProcessVariable");
				String xPath = getTransitionProperty(aTransition, "Expression");
				
				String newExpression = "#{xpath("+xPathVariable+","+xPath+")}";
				setPropertyOnTransition(aTransition, "Expression", newExpression);
				// ensure not execute the following code
				transitionLabel = null;
			}

			if (transitionLabel != null) {

				String taskStatusVariablePrefix = prefs.getProperty(DeployPreferenceConstants.P_WF_VARIABLE_FOR_TASK_STATUS_PREFIX);
				String subProcessVariablePrefix = prefs.getProperty(DeployPreferenceConstants.P_WF_VARIABLE_FOR_SUBPROCESS_PREFIX);

				Step transitionSource = aTransition.getSource();
				Service serviceModel = transitionSource.getServiceModel();

				// DEVO CONTROLLARE IL NODO DA CUI LA MIA TRANSIZIONE PROVIENE
				// HO DUE CASI

				// 1 - UTENTE HA DISEGNATO L'IF IN MODO SPECIFICO ALLORA DEVO PRENDERE
				// IL NODO PRIMA DELL'IF
				if (getStepProperty(transitionSource, ImConstants.IS_EXCLUSIVE_GATEWAY) != null) {
					// IL VERO TRANSITION SOURCE E IL NODO PRIMA DELL'IF
					logger.error(transitionSource.getName() + " gateway ");
					// PRENDO LA LISTA DELLE TRANSITION DI CUI IL ROUTER E' TARGET
					EList<Transition> incomingTransition = transitionSource.getTargetTransitions();

					if ((incomingTransition != null) && (incomingTransition.size() > 0))
						transitionSource = incomingTransition.get(0).getSource();

				}

				// 2 - UTENTE HA DISEGNATO SENZA METTERE L'IF MI BASTA CONSIDERARE IL
				// SOURCE DELLA PRIMA INCOMING TRANSITION
				// CHE PUO' ESSERE UN TASK NORMALE O UN SOTTOPROCESSO

				// Inizializzo una variabile temporanea per il nome della variabile
				// usata nella transizione
				String variableBindOnConditionLocal = variableBindOnCondition;
				if (variableBindOnCondition.equalsIgnoreCase("$TaskInstanceVariableStatus")) {
					// VEDO SE IL TASK SOURCE DELLA TRANSAZIONE E UN SOTTO PROCESSO
					// IN QUESTO CASO LA TRANSIZIONE E CONDIZIONATA AL RISULTATO DEL
					// SOTTOPROCESSO
					//
					if (serviceModel != null && serviceModel.getServiceName().equalsIgnoreCase(SERVICE_SubprocessService)) {
						String subProcessName = getStepProperty(transitionSource, "SubProcess");
						variableBindOnConditionLocal = getVariableName(subProcessVariablePrefix, subProcessName);

					} else {
						variableBindOnConditionLocal = getVariableName(taskStatusVariablePrefix, transitionSource.getName());
					}
				}

				String elExpression = "#{" + variableBindOnConditionLocal + "=='" + transitionLabel + "'}";

				setPropertyOnTransition(aTransition, "Expression", elExpression);
				setPropertyOnTransition(aTransition, "ExpressionLanguage", "JSPEL");
			}
		}
	}

	public void createServiceBindingForRuntime(StpIntermediateModel im, String serviceBindingName) {

		ServiceBinding serviceBinding = factory.createServiceBinding();
		serviceBinding.setName(serviceBindingName);
		BasicProperty runtimeIdProperty = factory.createBasicProperty();
		runtimeIdProperty.setKey(ImConstants.IM_STEP_BINDING_RUNTIME_ID);
		runtimeIdProperty.setValue(getRuntimeId());
		serviceBinding.getProperties().put(runtimeIdProperty.getKey(), runtimeIdProperty);
		im.getServicebindings().add(serviceBinding);

	}

	public String getRuntimeId() {
		return runtimeId;
	}

	public void setRuntimeId(String runtimeId) {
		this.runtimeId = runtimeId;
	}

	public String getBasicPropertyValue(ConfigurableElement ce, String propertyName) {
		Property p = ce.getProperties().get(propertyName);
		if (p != null) {
			if (p instanceof BasicProperty) {
				return ((BasicProperty) p).getValue();
			} else {
				return p.toString();
			}
		} else {
			return null;
		}
	}

	public ServiceBinding getRightStepBinding(Step s) {
		Property runtimPropertyForServiceBinding = null;
		for (ServiceBinding sb : s.getStepbindings()) {
			runtimPropertyForServiceBinding = sb.getProperties().get(ImConstants.IM_STEP_BINDING_RUNTIME_ID);
			if (runtimPropertyForServiceBinding != null) {
				if (runtimPropertyForServiceBinding.getValue().equals(getRuntimeId())) {
					return sb;
				}
			} else {
				logger.warn("ServiceBinding[" + sb.getName() + "] -> for Step[" + s.getName()
						+ "] does not have a property called " + ImConstants.IM_POOL_RUNTIME_ID);
			}
		}
		logger.warn("Not found for Step[" + s.getName() + "] a binding for runtime "
				+ ImConstants.IM_POOL_RUNTIME_ID);
		return null;
	}

	public void setSignalEndpointURIOnStep(Process process, Step targetStep, Step signalStep) {
		setPropertyOnStep(targetStep, "signalEndpointURI", DeploymentUtils.getStepNameInProcess(process, signalStep));
	}

	public boolean isServiceBinding(Step step, String serviceBindingName) {
		return isServiceBinding(getRightStepBinding(step), serviceBindingName);
	}

	// Copies on the fly the Orbeon Forms URL in the process properties, collecting it in
	// the default preference store
	public void createOrbeonIntegrationProperty(Process process) {
		BasicProperty orbeonIntegrationProperty = factory.createBasicProperty();
		orbeonIntegrationProperty.setKey(
				ORBEON_INTEGRATON_PARAMETER);
		

		orbeonIntegrationProperty.setValue(prefs.getProperty(DeployPreferenceConstants.P_WF_SPAGIC_ORBEON_FORM_RUNNER_URL));

		
		process.getProperties().put(orbeonIntegrationProperty.getKey(),
				orbeonIntegrationProperty);
	}
	
	

}
