/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.connectors.tcp;

import java.util.Vector;
import java.util.concurrent.atomic.AtomicBoolean;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * This is a simple monitor to serialize the calls to sendTCPMessage()
 * in class TCPProviderEndpoint. It implements a mutex where waiting
 * processes form a FIFO queue so the first process awakened is the
 * first process that was put to sleep when it found that the mutex
 * was busy.
 *
 * Created: Tue Oct 27 11:22:02 2009
 *
 * @author boccalon/Gian Uberto Lauri
 */
public class FIFOMonitor {

	private static final Logger log = LoggerFactory.getLogger(FIFOMonitor.class);

	/**
	 * Creates a new instance of <code>FIFOMonitor</code> .
	 *
	 */
	public FIFOMonitor() {

	}

	private Vector<Thread> waitingQueue = new Vector<Thread>();
	private AtomicBoolean sectionIsBusy = new AtomicBoolean(false);

	
	/**
	 * <code>enterMonitor</code> is the method to call to enter the mutex
	 * section represented by this instance of FIFOMonitor.java. If the
	 * section is blocked, the thread is put to wait on its Thread instance
	 * and the instance is queued into the monitor waiting queue.
	 * @param s 
	 *
	 */
	// Believe me, this method has not to be synchronized or you get a nice
	// deadlock with the other method. The Vector methods is syncronized, 
	// so it gives no race.
	public void enterMonitor(String s) {
		while(sectionIsBusy.getAndSet(true)) {
				Thread t = Thread.currentThread();
				waitingQueue.add(t);
				log.debug(s + " thread id queues " + t.getId());
				try {
					synchronized(t){
						t.wait();
					}
				}
				catch(InterruptedException ex) {
				}
			}
	}

	/**
	 * <code>leaveMonitor</code> this is the code to call to leave the
	 * mutex section. The thread resets the "busy" state of the section and
	 * awakens the first thread in the waiting queue.
	 *
	 */
	// Believe me, this method has not to be synchronized or you get a nice
	// deadlock with the other method. The Vector method is synchronized, so 
	// it gives no race.
	public void leaveMonitor(String s) {
							   
			if(waitingQueue.size() > 0) {
				Thread t = waitingQueue.remove(0);

				synchronized(t){
					sectionIsBusy.set(false);
					log.debug(s + " thread id notified " + t.getId());
					t.notify();
				}

			}
			else
				sectionIsBusy.set(false);

	}

}
