/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.exchanges;

import java.sql.Blob;
import java.sql.SQLException;

import org.eclipse.ebpm.messaging.api.Exchange;
import org.hibernate.Criteria;
import org.hibernate.Hibernate;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.cfg.Configuration;
import org.hibernate.criterion.Expression;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.spagic3.exchanges.model.ExchangeWrapper;

/**
 * Class <code>DbExchangeStore</code> provides an implementation of
 * IExchangeStore that uses the database (a table in the metadata DB).
 *
 * @author <a href="mailto:saint@eng.it">Gian Uberto Lauri</a>
 * @version $Revision$
 */
public class DbExchangeStore implements IExchangeStore {

	private static final Logger logger = 
		LoggerFactory.getLogger(DbExchangeStore.class);

	private final ExchangeSerializer xchngsrlzr;

	private final SessionFactory sessionFactory;

	/**
	 * Creates a new instance of <code>DbExchangeStore</code> .
	 *
	 */
	public DbExchangeStore() {
		xchngsrlzr = new ExchangeSerializer();

		try {
			sessionFactory = new Configuration().configure()
					.buildSessionFactory();
		} catch (Throwable ex) {
			// Make sure you log the exception, as it might be swallowed
			logger.error("Initial SessionFactory creation failed.", ex);
			throw new ExceptionInInitializerError(ex);
		}
	}

	/**
	 * <code>store</code> persists an Exchange object
	 *
	 * @param pid a <code>Long</code> id of the Exchange object to be
	 * persisted
	 * @param ex an <code>Exchange</code> the Exchange object to be
	 * persisted
	 */
	@Override
	public void store(Long pid, Exchange ex) {
		Session session = sessionFactory.openSession();
		ExchangeWrapper exw = new ExchangeWrapper();

		exw.setData(Hibernate.createBlob(xchngsrlzr.toByteArray(ex) ));
		exw.setProcessId(pid);

		Transaction transact = session.beginTransaction();

		try {
			session.save(exw);
			transact.commit();
		}
 		catch (Throwable rex) {
 			try {
 				transact.rollback();
 			}
			catch (Throwable thr) {
				logger.error("Unexpected Throwable doing rollback on insert - swallowed.",thr);
			}
 			logger.error("RuntimeException handling the BLOB",rex);
 			throw new RuntimeException("Unexpected Throwable handling the BLOB",rex);
 		}
		finally {
			try {
				session.close();
			}
			catch(HibernateException hex) {
				logger.error("Session close failure",hex);
			}
		}

	}

	/**
	 * <code>load</code> retrieves an Exchange object from its pid
	 *
	 * @param pid a <code>Long</code> id of the Exchange object to be
	 * retrieved
	 * @return an <code>Exchange</code> the Exchange object retrieved
	 */
	@Override
	public Exchange load(Long pid) {
		Session session = sessionFactory.openSession();
		Exchange result = null;
		
		Transaction transaction = session.beginTransaction();
		try {
			transaction = session.beginTransaction();
			// display of erudtion ;) :) :) :) : )
			Criteria aliquaCriteria = session.createCriteria(ExchangeWrapper.class);
			aliquaCriteria.add(Expression.eq("processId",pid));

			ExchangeWrapper exw = (ExchangeWrapper) aliquaCriteria.uniqueResult();

			Blob b = exw.getData();
			int blen = (int) b.length();
			result = xchngsrlzr.fromByteArray(b.getBytes(1, blen));

			transaction.commit();
			transaction = null;

			return result;
		} 
		catch (HibernateException hex) {
 			try {
 				if(transaction != null)
 					transaction.rollback();
 			}
			catch (Throwable thrRollback) {
				logger.error("Unexpected Throwable doing rollback on select - swallowed.",thrRollback);
			}
			logger.error("SQLException handling the BLOB",hex);
			throw new RuntimeException("HibernateException handling the BLOB",hex);
		} catch (SQLException sex) {
 			try {
 				if(transaction != null)
 					transaction.rollback();
 			}
			catch (Throwable thrRollback) {
				logger.error("Unexpected Throwable doing rollback on select - swallowed.",thrRollback);
			}
			logger.error("SQLException handling the BLOB",sex);
			throw new RuntimeException("SQLException handling the BLOB",sex);
		} 
		catch (Exception sex) {
 			try {
 				if(transaction != null)
 					transaction.rollback();
 			}
			catch (Throwable thrRollback) {
				logger.error("Unexpected Throwable doing rollback on select - swallowed.",thrRollback);
			}
			logger.error("SQLException handling the BLOB",sex);
			throw new RuntimeException("SQLException handling the BLOB",sex);
		} 
		finally {
			try {
				session.close();
			}
			catch(HibernateException hex) {
				logger.error("Session close failure",hex);
			}
		}

	}

	/**
	 * <code>delete</code> retrieves and deletes an Exchange object
	 * from its pid
	 *
	 * @param pid a <code>Long</code> id of the Exchange object to be
	 * retrieved and deleted to be persisted
	 * @return an <code>Exchange</code> the Exchange object retrieved
	 * and deleted to be persisted
	 */
	@Override
	public Exchange delete(Long pid) {
		Exchange result = load(pid);
		
		Session session = sessionFactory.openSession();
		Transaction transact = null;

		try {
			transact = session.beginTransaction();
			Query killer = session.createQuery("delete from ExchangeWrapper where processId = :exid");
			killer.setLong("exid", pid);
			killer.executeUpdate();
			transact.commit();

		}
		catch(HibernateException hex) {
			logger.error("Attachment deletion failure",hex);
			if(transact !=null)
				transact.rollback();
			throw new RuntimeException("Attachment deletion failure",hex);
		}
		catch(Throwable thr) {
			logger.error("Attachment deletion failure - Unexpected exception",thr);
			if(transact !=null)
				transact.rollback();
			throw new RuntimeException("Attachment deletion failure - Unexpected exception",thr);
		}
		finally {
			try {
				session.close();
			}
			catch(HibernateException hex) {
				logger.error("Session close failure - swallowed",hex);
			}
		}
		return result;
	}

}
